package com.cc.tools.activity

import android.content.Intent
import android.os.Bundle
import android.support.v4.app.Fragment
import android.support.v7.app.AppCompatActivity
import android.view.MotionEvent
import com.cc.tools.R
import com.cc.tools.fragment.base.BasicFragment
import com.cc.tools.model.common.ICodeModel
import com.cc.tools.utils.SystemBarUtils
import com.cc.tools.utils.start.ModuleUtils

/**
 */
open class BaseActivity : AppCompatActivity(), ICodeModel {

    private lateinit var mFragment: BasicFragment<*>

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        window.decorView.setBackgroundResource(R.drawable.tools_common_bg)
        val ft = supportFragmentManager.beginTransaction()
        val bundle = intent.extras
        val fName = bundle?.getString(ModuleUtils.F_CLASS)
        mFragment = Fragment.instantiate(this, fName, bundle) as BasicFragment<*>
        mFragment.lazyLoad()
        ft.add(android.R.id.content, mFragment)
        ft.commitAllowingStateLoss()
        SystemBarUtils.attachActivity(this)
        Intent.ACTION_USER_UNLOCKED
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent) {
        super.onActivityResult(requestCode, resultCode, data)
        mFragment?.onActivityResult(requestCode, resultCode, data)
    }

    override fun onResume() {
        super.onResume()
    }

    override fun onPause() {
        super.onPause()
    }

    override fun onBackPressed() {
        if (mFragment?.onBackPressed()) {
            return;
        }
        super.onBackPressed()
    }

    override fun onNewIntent(intent: Intent?) {
        super.onNewIntent(intent)
        if (intent != null && intent.extras != null) {
            mFragment?.arguments?.putAll(intent.extras)
        }
    }

    override fun dispatchTouchEvent(ev: MotionEvent): Boolean {
        mFragment.dispatchTouchEvent(ev)
        return super.dispatchTouchEvent(ev)
    }

    override fun sendCode() {
        mFragment.sendCode()
    }
}
