package com.cc.tools.application

import android.app.Application
import com.cc.tools.network.HttpClientInst
import com.cc.tools.network.HttpConstants
import com.cc.tools.network.download.DownloadManager
import com.cc.tools.utils.*
import com.cc.tools.utils.user.UserManager
import okhttp3.Request
import org.json.JSONException
import org.json.JSONObject

class BaseApplication : Application() {

    override fun onCreate() {
        super.onCreate()
        mApplication = this
        UserManager.inst.initUserManage(this)
        SystemBarUtils.initApplication(this)
        ViewUtils.ins.initApplication(this)
        ChannelManage.getInstance().initApplication(this)
        MachineInfoUtil.instance.initInfo(this)
        GlideUtils.ins.initApplication(this)
        DownloadManager.inst.initApplication(this)
        HttpClientInst.inst.setOnAddHeaderAction(object : HttpClientInst.OnAddHeaderAction {

            override fun addHeaderMap(builder: Request.Builder) {
                builder.addHeader(HttpConstants.HEADER_EVENT, MachineInfoUtil.instance.clientHeader)
                val jsonObject = JSONObject()
                try {
                    jsonObject.put("authorization_token", UserManager.inst.userToken)
                } catch (e: JSONException) {
                    e.printStackTrace()
                }
                builder.addHeader(HttpConstants.HEADER_USER_INFO, jsonObject.toString())
            }
        })
    }

    companion object {
        lateinit var mApplication: BaseApplication
    }
}
