package com.cc.tools.dlg

import android.app.AlertDialog
import android.content.Context
import android.os.Bundle
import android.os.Handler
import android.view.View
import android.widget.LinearLayout
import com.cc.tools.utils.HandlerUtils
import com.cc.tools.widget.layout.DlgLinearLayout


abstract class BaseAlertDialog : AlertDialog {

    var mContext: Context
    var mHandler: Handler = Handler()
    var mConvertView: View? = null
    var mHasParent: Boolean = false
    var mDlgLinearLayout: DlgLinearLayout? = null

    /**
     * 对话框的ID
     *
     * @return
     */
    abstract val layoutRes: Int

    constructor(context: Context) : super(context) {
        this.mHasParent = true
        this.mContext = context
    }

    constructor(context: Context, theme: Int) : super(context, theme) {
        this.mContext = context
    }

    abstract fun initViews(convertView: View)

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (mHasParent) {
            mDlgLinearLayout = DlgLinearLayout(mContext)
            mConvertView = layoutInflater.inflate(layoutRes, mDlgLinearLayout)
            setContentView(mDlgLinearLayout)
        } else {
            val linearLayout = LinearLayout(mContext)
            mConvertView = layoutInflater.inflate(layoutRes, linearLayout)
            setContentView(linearLayout)
        }
        initViews(mConvertView!!)
    }

    fun showDialog(): BaseAlertDialog {
        show()
        return this
    }

    override fun dismiss() {
        super.dismiss()
        HandlerUtils.removeCallbacksAndMessages(mHandler)
    }
}
