package com.cc.tools.fragment

import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import butterknife.BindView
import butterknife.OnClick
import com.cc.tools.R
import com.cc.tools.fragment.base.TitleFragment
import com.cc.tools.model.common.ICCModel
import com.cc.tools.observer.APkInstalledObserver
import com.cc.tools.presenter.common.ICCPresenter
import com.cc.tools.utils.HandlerUtils

class CCFragment : TitleFragment<ICCPresenter>, ICCModel {

    @BindView(R.id.layout_cc_down_btn)
    lateinit var btn: TextView

    var mListener: OnCCFragmentListener? = null

    constructor() {
        APkInstalledObserver.inst.addListener(this)
        mTitleLayoutRes = 0
        mTitleRes = R.string.text_register
        mLayoutRes = R.layout.layout_cc_down
        mIPresenterClass = ICCPresenter::class
    }

    override fun initView(inflater: LayoutInflater?, container: View?) {
        super.initView(inflater, container)
    }

    override fun hideSelf() {
        activity.supportFragmentManager.beginTransaction().hide(this).commit()
    }

    override fun showSelf() {
        activity.supportFragmentManager.beginTransaction().show(this).commit()
    }

    override fun setDownloadStatus(status: Int) {
        HandlerUtils.post(mHandler, Runnable {
            when (status) {
                ICCModel.TYPE_IDE -> {
                    btn.text = "下载"
                }
                ICCModel.TYPE_DOWN_ING -> {
                    btn.text = "下载中"
                }
                ICCModel.TYPE_DOWN_ED -> {
                    btn.text = "安装"
                }
                ICCModel.TYPE_DOWN_PAUSE -> {
                    btn.text = "继续"
                }
                ICCModel.TYPE_INSTALL_ING -> {
                    btn.text = "安装中"
                }
                ICCModel.TYPE_INSTALL_ED -> {
                    btn.text = "安装"
                }
            }
        })
    }

    @OnClick(R.id.layout_cc_down_btn)
    fun onDownloadClick() {
        mIPresenterImp?.onclick(activity)
    }

    override fun onResume() {
        super.onResume()
        mIPresenterImp?.installApp(ICCPresenter.CC_PKG)
    }

    override fun installApp(pkg: String) {
        mIPresenterImp?.installApp(pkg)
    }

    override fun uninstallApp(pkg: String) {
        mIPresenterImp?.uninstallApp(pkg)
    }

    override fun onHiddenChanged(hidden: Boolean) {
        super.onHiddenChanged(hidden)
        mListener?.onCCHiddenChanged(hidden)
    }

    interface OnCCFragmentListener {

        fun onCCHiddenChanged(hidden: Boolean)

    }

    override fun onDestroy() {
        super.onDestroy()
        APkInstalledObserver.inst.removeListener(this)
    }
}