package com.cc.tools.fragment

import android.text.*
import android.text.style.AbsoluteSizeSpan
import android.text.style.ForegroundColorSpan
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import butterknife.BindView
import butterknife.OnClick
import com.cc.tools.R
import com.cc.tools.bean.CacheBean
import com.cc.tools.bean.ConfigBean
import com.cc.tools.bean.user.UserInfoBean
import com.cc.tools.dlg.DlgCacheCover
import com.cc.tools.dlg.DlgCacheNew
import com.cc.tools.dlg.DlgUpload
import com.cc.tools.dlg.DlgZip
import com.cc.tools.fragment.base.TitleFragment
import com.cc.tools.model.IMainHeaderModel
import com.cc.tools.network.IProtocolListener
import com.cc.tools.network.ResponseBean
import com.cc.tools.network.protocol.config.ProtocolComCcToolsLoad
import com.cc.tools.observer.APkInstalledObserver
import com.cc.tools.observer.UserLoginObserver
import com.cc.tools.observer.UserLogoutObserver
import com.cc.tools.presenter.IMainHeaderPresenter
import com.cc.tools.utils.*
import com.cc.tools.utils.start.LoginModuleUtils
import com.cc.tools.utils.user.UserManager
import com.cc.tools.widget.text.UserDownloadText
import java.text.SimpleDateFormat
import java.util.*

class MainHeaderFragment : TitleFragment<IMainHeaderPresenter>, IMainHeaderModel {


    val MAX_LEGNTH = 5
    val TYPE_NEW = -2
    val TYPE_NONE = -1

    @BindView(R.id.fragment_main_header_icon)
    lateinit var mUserIcon: ImageView
    @BindView(R.id.fragment_main_header_name)
    lateinit var mNameView: TextView
    @BindView(R.id.fragment_main_header_btn)
    lateinit var mLoginBtn: TextView
    @BindView(R.id.fragment_main_header_cache_cover)
    lateinit var mCoverTv: TextView
    @BindView(R.id.fragment_main_header_notice)
    lateinit var mNoticeTv: TextView
    @BindView(R.id.fragment_main_header_notice_cover)
    lateinit var mNoticeCover: TextView
    @BindView(R.id.fragment_main_header_cache_content_layout)
    lateinit var mCacheUserLayout: ViewGroup
    @BindView(R.id.down_now_layout)
    lateinit var mDownNowLayout: ViewGroup
    @BindView(R.id.down_now_text)
    lateinit var mdown_now_text: TextView

    var mConfigBean = ConfigBean()
    var mHasCache = false

    var mDlgCacheNew: DlgCacheNew? = null
    var mDlgCacheCover: DlgCacheCover? = null
    var mDlgZip: DlgZip? = null
    var mDlgUpload: DlgUpload? = null
    var mCacheBean: CacheBean? = null
    var mSelectView: UserDownloadText? = null
    var mLength = 0

    constructor() {
        APkInstalledObserver.inst.addListener(this)
        UserLoginObserver.inst.addListener(this)
        UserLogoutObserver.inst.addListener(this)
        mTitleLayoutRes = 0
        mTitleRes = R.string.text_register
        mLayoutRes = R.layout.fragment_main_header
        mIPresenterClass = IMainHeaderPresenter::class
    }

    override fun initView(inflater: LayoutInflater?, container: View?) {
        super.initView(inflater, container)
        if (UserManager.inst.isLogin) {
            onLoginSuccess()
        } else {
            onLogOutSuccess()
        }
        noticeCover()
        var protocol = ProtocolComCcToolsLoad()
        protocol.mContext = activity
        protocol.mListener = object : IProtocolListener<ConfigBean> {
            override fun onStart() {
            }

            override fun onSuccess(bean: ResponseBean<ConfigBean>) {
                if (bean != null && bean.results != null) {
                    mConfigBean = bean.results!!
                }
            }

            override fun onFailure(bean: ResponseBean<ConfigBean>) {
            }

            override fun onFinish(bean: ResponseBean<ConfigBean>) {
                mdown_now_text.text = mConfigBean.packageTitle
            }
        }
        protocol.postRequest()
    }

    @OnClick(R.id.fragment_main_header_btn)
    fun onLoginClick() {
        if (UserManager.inst.isLogin) {
            UserManager.inst.logout()
        } else {
            LoginModuleUtils.startLoginActivity(activity)
        }
    }

    //存档
    @OnClick(R.id.fragment_main_header_cache_cover)
    fun onCacheCover() {
        if (UserManager.inst.isLogin) {
            if (checkFileExit()) {
                return
            }
            if (mHasCache) {
                if (mCacheBean != null) {
                    var builder = SpannableStringBuilder()
                    builder.append("是否覆盖存档：")
                    var ss = SpannableString(mCacheBean!!.archiveName)
                    ss.setSpan(ForegroundColorSpan(resources.getColor(R.color.color_common_red)), 0, ss.length, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
                    builder.append(ss)
                    showDlgCacheCover(builder, mCacheBean!!.id)
                } else {
                    if (mLength < MAX_LEGNTH) {
                        showDlgCacheNew("", -1)
                    } else {
                        ToastUtils.showLongToast(activity, "请选择一个存档覆盖")
                    }
                }
            } else {
                showDlgCacheNew("", -1)
            }
        } else {
            LoginModuleUtils.startLoginActivity(activity)
        }
    }

    @OnClick(R.id.down_now)
    fun onDownloadNow() {
        AppManager.openUrl(activity, mConfigBean.downloadUrl)
    }

    fun checkFileExit(): Boolean {
        if (TextUtils.isEmpty(CacheUtils.getDQMR_3())) {
            ToastUtils.showLongToast(activity, "您还未安装游戏，请点击下方前往下载~")
            return true
        }
        return false
    }

    override fun onResume() {
        super.onResume()
        if (!TextUtils.isEmpty(CacheUtils.getDQMR_3())) {
            mDownNowLayout.visibility = View.GONE
        } else {
            mDownNowLayout.visibility = View.VISIBLE
        }
    }

    //读档
    @OnClick(R.id.fragment_main_header_cache_read)
    fun onCacheRead() {
        if (UserManager.inst.isLogin) {
            if (mHasCache) {
                if (mCacheBean == null) {
                    ToastUtils.showLongToast(activity, "请选择一个存档进行读取~")
                } else {
                    mSelectView?.readCache()
                }
            } else {
                ToastUtils.showLongToast(activity, "无存档可读取")
            }
        } else {
            LoginModuleUtils.startLoginActivity(activity)
        }
    }

    override fun onLoginSuccess() {
        setUserInfo(UserManager.inst.userInfo!!)
        mIPresenterImp?.getUserInfo()
        mIPresenterImp?.getUserCacheList()
    }

    override fun setUserInfo(userInfoBean: UserInfoBean) {
        if (UserManager.inst.isLogin) {
            mLoginBtn.text = resources.getString(R.string.text_logout)
            if (mHasCache) {
                mCoverTv.text = "覆盖存档"
            } else {
                mCoverTv.text = "存档"
            }
            mNoticeTv.text = "我的存档"
        } else {
            mHasCache = false
            mCoverTv.text = "存档"
            mNoticeTv.text = "登录可存档读档"
            mLoginBtn.text = resources.getString(R.string.text_header_login)
            mCacheUserLayout.removeAllViews()
        }
        GlideUtils.ins.loadUserIcon(activity, userInfoBean.icon, mUserIcon)
        mNameView.text = userInfoBean.nick_name
    }

    override fun onFirstRequestSuccess(data: ResponseBean<MutableList<CacheBean>>) {
        mCacheBean = null
        mHasCache = data.results!!.isNotEmpty()
        mLength = data.results!!.size
        mCacheUserLayout.removeAllViews()
        if (mHasCache) {
            mCoverTv.text = "覆盖存档"
            for (bean in data.results!!) {
                addItemView(bean)
            }
            if (data.results!!.size < MAX_LEGNTH) {
                var bean = CacheBean()
                bean.id = TYPE_NEW
                bean.archiveName = "上传新存档"
                addItemView(bean)
            }
        } else {
            mCoverTv.text = "存档"
            onFirstRequestFail(data)
        }
    }

    fun addItemView(bean: CacheBean) {
        var itemView = UserDownloadText(activity)
        when (bean.id) {
            TYPE_NEW, TYPE_NONE -> {
                itemView.text = bean.archiveName
            }
            else -> {
                var builder = SpannableStringBuilder()
                builder.append(bean.archiveName)
                builder.append("\n")
                var ss = SpannableString(formatSecondToMinute(bean.updateDatetime))
                ss.setSpan(AbsoluteSizeSpan(DisplayUtils.dip2px(context, 10f)), 0, ss.length, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
                builder.append(ss)
                itemView.text = builder
            }
        }
        itemView.gravity = Gravity.CENTER
        itemView.setBackgroundResource(R.drawable.drawable_selector_round)
        itemView.setTextColor(resources.getColorStateList(R.color.color_selector_white_2_yellow))
        itemView.textSize = 14f
        itemView.setPadding(DisplayUtils.dip2px(activity, 12f), 0, DisplayUtils.dip2px(activity, 12f), 0)
        itemView.checkDownloadStatus(bean)
        itemView.setOnClickListener {
            when (bean.id) {
                TYPE_NONE -> {
                    onCacheCover()
                }
                TYPE_NEW -> {
                    if (!checkFileExit()) {
                        showDlgCacheNew("", -1)
                    }
                }
                else -> {
                    mSelectView?.isSelected = false
                    mCacheBean = bean
                    it.isSelected = true
                    mSelectView = it as UserDownloadText
                }
            }
        }
        var layoutParams = LinearLayout.LayoutParams(-1, DisplayUtils.dip2px(activity, 43f))
        layoutParams.topMargin = DisplayUtils.dip2px(activity, 16f)
        mCacheUserLayout.addView(itemView, layoutParams)
    }

    override fun onFirstRequestFail(data: ResponseBean<MutableList<CacheBean>>) {
        mCacheBean = null
        var bean = CacheBean()
        bean.id = TYPE_NEW
        bean.archiveName = "上传新存档"
        addItemView(bean)
    }

    override fun onFirstRequestFinish(data: ResponseBean<MutableList<CacheBean>>) {
    }

    override fun onFirstRequestStart() {
    }

    override fun onLogOutSuccess() {
        mHasCache = false
        mCacheUserLayout.removeAllViews()
        mCacheBean = null
        mSelectView = null
        mLength = 0

        var bean = UserInfoBean()
        bean.nick_name = "请先登录\n可使用虫虫账号登录"
        setUserInfo(bean)
    }

    fun showDlgCacheNew(title: String, id: Int) {
        mDlgCacheCover?.dismiss()
        mDlgCacheNew = DlgCacheNew(activity)
        mDlgCacheNew!!.mTitle = title
        mDlgCacheNew!!.mListener = object : DlgCacheNew.OnDlgCacheListener {

            override fun onDone(name: String) {
                mIPresenterImp?.zipCache(name, id)
            }
        }
        mDlgCacheNew!!.show()
    }

    fun showDlgCacheCover(title: CharSequence, id: Int) {
        mDlgCacheCover?.dismiss()
        mDlgCacheCover = DlgCacheCover(activity)
        mDlgCacheCover!!.mCacheCoverNotice = title
        mDlgCacheCover!!.mListener = object : DlgCacheCover.OnDlgCacheCoverListener {

            override fun onSure() {
                showDlgCacheNew(resources.getString(R.string.dlg_cache_cover), id)
            }
        }
        mDlgCacheCover!!.show()
    }

    fun showDlgUnZip() {
        mDlgZip?.dismiss()
        mDlgZip = DlgZip(activity)
        mDlgZip?.show()
    }

    override fun onZipStart() {
        HandlerUtils.post(mHandler, Runnable {
            showDlgUnZip()
        })
    }

    override fun onZipError() {
        HandlerUtils.post(mHandler, Runnable {
            ToastUtils.showLongToast(activity, "文件压缩失败，请确认是否有存档文件~")
            mDlgZip?.dismiss()
        })
    }

    override fun onZipComplete() {
        HandlerUtils.post(mHandler, Runnable {
            ToastUtils.showLongToast(activity, "文件压缩成功~")
            mDlgZip?.dismiss()
        })
    }

    override fun onZipProgress(name: String) {
        HandlerUtils.post(mHandler, Runnable {
            mDlgZip?.setText(name)
        })
    }

    fun showDlgUpload() {
        mDlgUpload?.dismiss()
        mDlgUpload = DlgUpload(activity)
        mDlgUpload?.show()
    }

    override fun IUploadStart() {
        HandlerUtils.post(mHandler, Runnable {
            showDlgUpload()
        })
    }

    override fun IUploadError() {
        HandlerUtils.post(mHandler, Runnable {
            mDlgUpload?.dismiss()
        })
    }

    override fun IUploadProgress(progress: Long, total: Long) {
        HandlerUtils.post(mHandler, Runnable {
            mDlgUpload?.setPoint(progress, total)
        })
    }

    override fun IUploadSuccess() {
        HandlerUtils.post(mHandler, Runnable {
            mDlgUpload?.dismiss()
        })
    }

    fun formatSecondToMinute(time: Long): String {
        val dateFormat = SimpleDateFormat("yyyy-MM-dd HH:mm")
        if (time.toString().length < 13) {
            return dateFormat.format(Date(time * 1000))
        }
        return dateFormat.format(Date(time))
    }

    override fun installApp(pkg: String) {
        noticeCover()
    }

    override fun uninstallApp(pkg: String) {
        noticeCover()
    }

    fun noticeCover() {
        HandlerUtils.post(mHandler, Runnable {
            mNoticeCover.text = String.format("覆盖存档请保证程序完全退出，如果覆盖存档后打开" +
                    "游戏出现卡进度。请用文件管理器到SD卡/Android/%s下删除cache文件夹。如果还不行，" +
                    "请加虫虫官网讨论群：578149842，进行咨询", CacheUtils.getDQMR_2())
        })
    }

    override fun onDestroy() {
        super.onDestroy()
        mDlgCacheNew?.dismiss()
        mDlgCacheCover?.dismiss()
        mDlgZip?.dismiss()
        mDlgUpload?.dismiss()
        APkInstalledObserver.inst.removeListener(this)
        UserLoginObserver.inst.removeListener(this)
        UserLogoutObserver.inst.removeListener(this)
    }
}