package com.cc.tools.fragment.base

import android.os.Bundle
import android.os.Handler
import android.support.v4.app.Fragment
import android.view.LayoutInflater
import android.view.MotionEvent
import android.view.View
import android.view.ViewGroup
import butterknife.ButterKnife
import butterknife.Unbinder
import com.cc.tools.model.common.ICodeModel
import com.cc.tools.presenter.IPresenterImp
import kotlin.reflect.KClass

open abstract class BasicFragment<Presenter> : Fragment(), ICodeModel {

    var mContentView: View? = null
    var mLayoutRes: Int = 0

    var mIPresenterClass: Any? = null
    var mIPresenterImp: Presenter? = null

    var mUnBinder: Unbinder? = null
    var mLazyLoad = false

    var mHandler = Handler()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        var constructor = (mIPresenterClass as KClass<*>).java.getConstructor()
        var iPresenterImp = constructor?.newInstance() as IPresenterImp<Any>
        iPresenterImp.setIModelImp(this)
        mIPresenterImp = iPresenterImp as Presenter
        (mIPresenterImp as IPresenterImp<*>).setContext(activity)
    }

    override fun onCreateView(inflater: LayoutInflater?, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        if (mContentView != null) {
            val viewGroup = mContentView?.getParent() as ViewGroup
            viewGroup?.removeView(mContentView)
        } else {
            mContentView = inflater?.inflate(mLayoutRes, null)!!
            mUnBinder = ButterKnife.bind(this, mContentView!!)
            initBaseFragment(inflater, mContentView, savedInstanceState)
            val viewParent = mContentView?.getParent()
            if (viewParent != null) {
                mContentView = viewParent as View
            }
        }
        return mContentView
    }

    open fun initBaseFragment(inflater: LayoutInflater?, container: View?, savedInstanceState: Bundle?) {
        initView(inflater, container)
        initData()
        checkLoadData()
    }

    open fun initView(inflater: LayoutInflater?, container: View?) {}

    fun initData() {
        (mIPresenterImp as IPresenterImp<*>).initData()
    }

    fun lazyLoad() {
        if(!mLazyLoad){
            mLazyLoad = true
            checkLoadData()
        }
    }

    fun checkLoadData() {
        if (mContentView != null && mLazyLoad) {
            loadData()
        }
    }

    open fun loadData() {
        (mIPresenterImp as IPresenterImp<*>).loadData()
    }

    fun findViewById(resId: Int): View? {
        return mContentView!!.findViewById(resId)
    }

    open fun onBackPressed(): Boolean {
        return false
    }

    override fun onDestroy() {
        super.onDestroy()
        mUnBinder?.unbind()
    }

    open fun dispatchTouchEvent(ev: MotionEvent): Boolean {
        return false
    }

    open fun gotoTop() {
    }

}