package com.cc.tools.fragment.base

import android.graphics.PointF
import android.graphics.drawable.Drawable
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.LinearSmoothScroller
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import com.cc.tools.R
import com.cc.tools.model.common.IRequestImp
import com.cc.tools.model.common.IRequestNextImp
import com.cc.tools.network.ResponseBean
import com.cc.tools.presenter.IPresenterImp
import com.cc.tools.widget.reclyer.BaseViewAdapter
import com.cc.tools.widget.reclyer.CustomRecyclerView


open abstract class RecycleFragment<Presenter, T> : TitleFragment<Presenter>, IRequestImp<ResponseBean<MutableList<T>>>, IRequestNextImp<ResponseBean<MutableList<T>>> {

    var mCustomRecyclerView: CustomRecyclerView? = null
    var mBeans: MutableList<T> = mutableListOf()
    var mAdapter: BaseViewAdapter<T>? = null
    var mLayoutManager: LinearLayoutManager? = null
    var mSmoothScroll: Boolean = false

    var mNeedFoot = true
    var mRequestNext = false
    var mOrdering = ""
    var mPage = 1

    val layoutRes: Int
        get() = R.layout.layout_recycleview

    val recycleViewId: Int
        get() = R.id.layout_recycleview

    val loadingViewParentId: Int
        get() = R.id.layout_recycleview

    val swipeRefreshLayoutId: Int
        get() = R.id.layout_recycleview_pull

    constructor() {}

    val layoutManager: LinearLayoutManager
        get() = object : LinearLayoutManager(activity, LinearLayoutManager.VERTICAL, false) {
            override fun smoothScrollToPosition(recyclerView: RecyclerView, state: RecyclerView.State?, position: Int) {
                val linearSmoothScroller = object : LinearSmoothScroller(recyclerView.context) {
                    override fun calculateTimeForScrolling(dx: Int): Int {
                        var dx = dx
                        if (dx > 3000) {
                            dx = 3000
                        }
                        return super.calculateTimeForScrolling(dx)
                    }

                    override fun computeScrollVectorForPosition(targetPosition: Int): PointF? {
                        return mLayoutManager?.computeScrollVectorForPosition(targetPosition)
                    }
                }
                linearSmoothScroller.targetPosition = position
                startSmoothScroll(linearSmoothScroller)
            }
        }

    abstract val adapter: BaseViewAdapter<T>

    var mOnScrollListener: RecyclerView.OnScrollListener = object : RecyclerView.OnScrollListener() {

        override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
            super.onScrolled(recyclerView, dx, dy)
            this@RecycleFragment.onScrolled(recyclerView, dx, dy)
        }

        override fun onScrollStateChanged(recyclerView: RecyclerView?, newState: Int) {
            super.onScrollStateChanged(recyclerView, newState)
        }
    }

    var ordering: String
        get() = ""
        set(ordering) {
            mOrdering = ordering
        }


    val noDataString: String
        get() = ""

    override fun initView(inflater: LayoutInflater?, container: View?) {
        super.initView(inflater, container)
        mCustomRecyclerView = container?.findViewById(recycleViewId) as CustomRecyclerView
        mAdapter = adapter
        if (mAdapter != null) {
            mAdapter!!.setBeans(mBeans)
        }
        mLayoutManager = layoutManager
        mLayoutManager?.isSmoothScrollbarEnabled = mSmoothScroll
        mCustomRecyclerView!!.layoutManager = mLayoutManager
        mCustomRecyclerView!!.adapter = mAdapter as RecyclerView.Adapter<*>
        mCustomRecyclerView!!.setDividerHeight(0.5f)
        mCustomRecyclerView!!.setHorizontalDrawable(resources.getDrawable(R.color.color_common_line))
        addHeaderOrFooterViews(mCustomRecyclerView!!, inflater)

//        if (mNeedFoot) {
//            mFooterView = LayoutInflaterUtils.inflateView(mParent, R.layout.layout_listview_footerview) as FooterView
//            mCustomRecyclerView!!.addFooterView(mFooterView)
//            mFooterView!!.showFooterView(false)
//        }
    }

    fun setLargeDivider() {
        mCustomRecyclerView!!.setDividerHeightPx(getResources().getDimensionPixelSize(R.dimen.common_line_large))
    }

    override fun gotoTop() {
        if (mCustomRecyclerView != null) {
            mCustomRecyclerView!!.smoothScrollToPosition(0)
        }
    }

    fun setSmoothScroll(smoothScroll: Boolean) {
        this.mSmoothScroll = smoothScroll
    }

    open fun addHeaderOrFooterViews(recyclerView: CustomRecyclerView, inflater: LayoutInflater?) {

    }

    override fun onBackPressed(): Boolean {
        return mAdapter != null && mAdapter!!.onBackPressed() || super.onBackPressed()
    }

    fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
        val view = recyclerView.getChildAt(recyclerView.childCount - 1)
        if (mLayoutManager?.findLastVisibleItemPosition()!! >= recyclerView.adapter.itemCount - 2) {
//        if (view is FooterLayout) {
            onScrollLastPosition()
        }
    }

    fun onScrollLastPosition() {
        if (!mRequestNext) {
            mRequestNext = true
            getNextData()
        }
    }

    fun getNextData() {
        (mIPresenterImp as IPresenterImp<*>).getNextData(mPage, pageSize = 10)
    }

    fun setHorizontalDrawable(horizontalDrawable: Drawable) {
        if (mCustomRecyclerView != null) {
            mCustomRecyclerView!!.setHorizontalDrawable(horizontalDrawable)
        }
    }

    fun setVerticalDrawable(verticalDrawable: Drawable) {
        if (mCustomRecyclerView != null) {
            mCustomRecyclerView!!.setVerticalDrawable(verticalDrawable)
        }
    }

    fun setDividerHeight(dip: Float) {
        if (mCustomRecyclerView != null) {
            mCustomRecyclerView!!.setDividerHeight(dip)
        }
    }

    fun showFooterView(isShow: Boolean) {
//        if (mFooterView != null) {
//            mFooterView!!.showFooterView(isShow)
//        }
    }

    fun doOnRefresh() {
        mPage = 1
        showFooterView(false)
        mCustomRecyclerView!!.removeOnScrollListener(mOnScrollListener)
    }

    fun showNoDataOrHide() {
//        if (mBeans.isEmpty()) {
//            showNoData(noDataString)
//        } else {
//            hideLoadingLayout()
//        }
    }

    fun addOnScrollListener(b: Boolean) {
        if (b && mCustomRecyclerView != null) {
            mCustomRecyclerView!!.addOnScrollListener(mOnScrollListener)
        }
    }

    fun removeOnScrollListener(b: Boolean) {
        if (b && mCustomRecyclerView != null) {
            mCustomRecyclerView!!.removeOnScrollListener(mOnScrollListener)
        }
    }

    fun notifyItemChanged(size: Int) {
        if (size > 0) {
            mAdapter!!.notifyItemChanged(size)
        }
    }

    fun scrollToPositionWithOffset(position: Int, y: Int) {
        mLayoutManager?.scrollToPositionWithOffset(position, y)
    }


    override fun onFirstRequestStart() {
    }

    override fun onFirstRequestSuccess(data: ResponseBean<MutableList<T>>) {
        execHeaderView(data.results!!)
        mBeans.clear()
        mBeans.addAll(data.results!!)
        mAdapter!!.notifyDataSetChanged()
        addOnScrollListener(data.results!!.size == 10)
        showNoDataOrHide()
        mPage = 2
    }

    fun execHeaderView(beanList: List<T>) {
    }

    override fun onFirstRequestFail(data: ResponseBean<MutableList<T>>) {
        showLoadFail()
    }

    override fun onFirstRequestFinish(data: ResponseBean<MutableList<T>>) {
    }

    //-------------下一页请求---------------------------
    override fun onNextRequestStart() {
        showFooterView(true)
    }

    override fun onNextRequestSuccess(data: ResponseBean<MutableList<T>>) {
        mBeans.addAll(data.results!!)
        notifyItemChanged(data.results!!.size)
        mPage++
        removeOnScrollListener(data.results!!.size != 10)
        onNextRequestFinish(data)
    }

    override fun onNextRequestFail(data: ResponseBean<MutableList<T>>) {
        onNextRequestFinish(data)
    }

    override fun onNextRequestFinish(data: ResponseBean<MutableList<T>>) {
        showFooterView(false)
        mRequestNext = false
    }


}
