package com.cc.tools.fragment.base

import android.content.Context
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.text.TextUtils
import android.view.*
import android.widget.LinearLayout
import com.cc.tools.R
import com.cc.tools.model.common.IMenuModel
import com.cc.tools.utils.Logger
import com.cc.tools.widget.actionbar.ActionBasicLayout


open abstract class TitleFragment<Presenter> : BasicFragment<Presenter>(), IMenuModel {

    var mTitleLayoutRes: Int = 0
    var mLeftIcon: Int = R.drawable.tools_common_back
    var mTitle: CharSequence = ""
    var mTitleRes: Int = 0
    var mMenuRes: Int = 0
    var mTitleBgDrawable: Drawable? = null
    var mFragmentLayoutClass = LinearLayout::class.java
    var mToolBar: ActionBasicLayout? = null
    var mToolMenu: Int = 0

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setHasOptionsMenu(mToolMenu > 0)
    }

    override fun initView(inflater: LayoutInflater?, container: View?) {
        super.initView(inflater, container)
        if (mTitleLayoutRes > 0) {
            mToolBar = inflater?.inflate(mTitleLayoutRes, null) as ActionBasicLayout
            if (TextUtils.isEmpty(mTitle)) {
                mToolBar?.setTitle(mTitleRes)
            } else {
                mToolBar?.title = mTitle
            }
            mToolBar?.setLeftIcon(mLeftIcon)
            if (mMenuRes > 0) {
                mToolBar?.inflateMenu(mMenuRes)
            }
            var constructor = mFragmentLayoutClass.getConstructor(Context::class.java)
            var layout = constructor.newInstance(activity)
            layout.addView(mToolBar, ViewGroup.LayoutParams(-1, -2))
            layout.addView(container, ViewGroup.LayoutParams(-1, -1))
            if (layout.javaClass == LinearLayout::class.java) {
                layout.orientation = LinearLayout.VERTICAL
            } else {
                layout.bringChildToFront(mToolBar)
            }
            layout.setBackgroundDrawable(mTitleBgDrawable)
            mContentView = layout
        }
    }

    open fun showLoadFail() {

    }

    override fun onCreateOptionsMenu(menu: Menu?, inflater: MenuInflater?) {
        Logger.i("onCreateOptionsMenu >>> ")
        if (mToolMenu > 0) {
            inflater?.inflate(mToolMenu, menu)
            mToolBar?.onCreateOptionsMenu(menu, inflater)
        }
    }

    override fun sendCode() {
    }

}