package com.cc.tools.fragment.login

import android.view.LayoutInflater
import android.view.View
import android.widget.EditText
import butterknife.BindView
import butterknife.OnClick
import com.cc.tools.R
import com.cc.tools.fragment.base.TitleFragment
import com.cc.tools.model.login.ILoginModel
import com.cc.tools.observer.UserLoginObserver
import com.cc.tools.presenter.login.ILoginPresenter
import com.cc.tools.utils.IMMUtils
import com.cc.tools.utils.ViewUtils
import com.cc.tools.utils.start.LoginModuleUtils

class LoginFragment : TitleFragment<ILoginPresenter>, ILoginModel {

    @BindView(R.id.fragment_login_phone)
    lateinit var mPhoneEt: EditText
    @BindView(R.id.fragment_login_pwd)
    lateinit var mPwdEt: EditText

    constructor() {
        UserLoginObserver.inst.addListener(this)
        mTitleLayoutRes = R.layout.actionbar_title_layout
        mTitleRes = R.string.text_login
        mLayoutRes = R.layout.fragment_login
        mIPresenterClass = ILoginPresenter::class
        mToolMenu = R.menu.login_menu
    }

    @OnClick(R.id.fragment_login_btn)
    fun gotoLogin() {
        var phone = mPhoneEt.text.toString().replace(" ", "")
        var pwd = mPwdEt.text.toString()
        if (!ViewUtils.ins.checkAccount(mPhoneEt, phone)) {
            return
        }
        if (!ViewUtils.ins.checkPwd(mPwdEt, pwd)) {
            return
        }
        IMMUtils.hideSoftInput(activity, mPhoneEt)
        IMMUtils.hideSoftInput(activity, mPwdEt)
        mIPresenterImp?.gotoLogin(phone, pwd)
    }

    @OnClick(R.id.fragment_login_register)
    fun onRegisterClick() {
        LoginModuleUtils.startRegisterActivity(activity)
    }

    @OnClick(R.id.fragment_login_forgot)
    fun onForgotPwd() {
        LoginModuleUtils.startPwdResetActivity(activity)
    }

    override fun onLoginSuccess() {
        activity.finish()
    }

    override fun initView(inflater: LayoutInflater?, container: View?) {
        super.initView(inflater, container)
    }

//    override fun onCreateOptionsMenu(menu: Menu?, inflater: MenuInflater?) {
//        super.onCreateOptionsMenu(menu, inflater)
//    }
//
//    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
//        LoginModuleUtils.startPwdResetActivity(activity)
//        return super.onOptionsItemSelected(item)
//    }

    override fun onDestroy() {
        super.onDestroy()
        UserLoginObserver.inst.removeListener(this)
    }
}