package com.cc.tools.fragment.login

import android.widget.EditText
import butterknife.BindView
import butterknife.OnClick
import com.cc.tools.R
import com.cc.tools.bean.VerImgCodeBean
import com.cc.tools.dlg.DlgVerify
import com.cc.tools.fragment.base.TitleFragment
import com.cc.tools.model.login.IRegisterModel
import com.cc.tools.network.ResponseBean
import com.cc.tools.observer.CodeTimeObserver
import com.cc.tools.observer.UserLoginObserver
import com.cc.tools.presenter.login.IRegisterPresenter
import com.cc.tools.utils.IMMUtils
import com.cc.tools.utils.ToastUtils
import com.cc.tools.utils.ViewUtils
import com.cc.tools.widget.input.CodeView

class RegisterFragment : TitleFragment<IRegisterPresenter>, IRegisterModel {

    @BindView(R.id.fragment_register_phone)
    lateinit var mPhoneEt: EditText
    @BindView(R.id.fragment_register_code)
    lateinit var mCodeEt: CodeView
    @BindView(R.id.fragment_register_pwd)
    lateinit var mPwdEt: EditText
    var mDlgVerify: DlgVerify? = null

    constructor() {
        UserLoginObserver.inst.addListener(this)
        mTitleLayoutRes = R.layout.actionbar_title_layout
        mTitleRes = R.string.text_register
        mLayoutRes = R.layout.fragment_register
        mIPresenterClass = IRegisterPresenter::class
    }

    @OnClick(R.id.fragment_register_commit)
    fun gotoCommit() {
        var phone = mPhoneEt.text.toString().replace(" ", "")
        var code = mCodeEt.text.toString()
        var pwd = mPwdEt.text.toString()
        if (!ViewUtils.ins.checkMobile(mPhoneEt, phone)) {
            return
        }
        if (!ViewUtils.ins.checkAuthCode(mCodeEt, code)) {
            return
        }
        if (!ViewUtils.ins.checkPwd(mPwdEt, pwd)) {
            return
        }
        IMMUtils.hideSoftInput(activity, mPhoneEt)
        IMMUtils.hideSoftInput(activity, mCodeEt)
        IMMUtils.hideSoftInput(activity, mPwdEt)
        mIPresenterImp?.gotoCommit(phone, code, pwd)
    }

    override fun sendCode() {
        var phone = mPhoneEt.text.toString().replace(" ", "")
        if (!ViewUtils.ins.checkMobile(mPhoneEt, phone)) {
            return
        }
        CodeTimeObserver.instance.startAnimation()
        mIPresenterImp?.sendCode(phone, "")
    }

    override fun showDlgVerify(phone: String, verifyImgUrl: String) {
        mDlgVerify = DlgVerify(activity)
        mDlgVerify!!.mPhone = phone
        mDlgVerify!!.mVerifyImgUrl = verifyImgUrl
        mDlgVerify!!.mOnCommitVerifyCodeAction = object : DlgVerify.OnCommitVerifyCodeAction {
            override fun onDone(phone: String, code: String) {
                mIPresenterImp?.sendCode(phone, code)
            }
        }
        mDlgVerify!!.show()
    }

    override fun hideDlgVerify() {
        mDlgVerify?.dismiss()
    }

    override fun sendCodeSuccess() {
        ToastUtils.showLongToast(activity, R.string.toast_send_security_code_success)
    }

    override fun refreshVerifyCode(bean: ResponseBean<VerImgCodeBean>) {
        if (mDlgVerify != null && mDlgVerify!!.isShowing) {
            mDlgVerify!!.refreshVerifyCode()
            ToastUtils.showLongToast(activity, bean.msg)
        } else {
            ToastUtils.showLongToast(activity, R.string.toast_send_security_code_fail)
        }
    }

    override fun onLoginSuccess() {
        activity.finish()
    }

    override fun onDestroy() {
        super.onDestroy()
        hideDlgVerify()
    }
}