package com.cc.tools.network

import com.cc.tools.network.upload.ProgressRequestBody
import com.cc.tools.utils.Logger
import com.qiniu.android.utils.AsyncRun
import okhttp3.*
import java.io.ByteArrayOutputStream
import java.io.File
import java.io.IOException
import java.io.RandomAccessFile
import java.util.concurrent.TimeUnit

class HttpClientInst private constructor() {

    private var mAction: OnAddHeaderAction? = null

    private//设置读取超时时间
            //设置写的超时时间
    val okHttpClient: OkHttpClient
        get() = OkHttpClient.Builder()
                .addInterceptor { chain ->
                    val builder = chain.request().newBuilder()
                    if (mAction != null) {
                        mAction!!.addHeaderMap(builder)
                    }
                    chain.proceed(builder.build())
                }
                .readTimeout(READ_TIMEOUT.toLong(), TimeUnit.SECONDS)
                .writeTimeout(WRITE_TIMEOUT.toLong(), TimeUnit.SECONDS)
                .connectTimeout(CONNECT_TIMEOUT.toLong(), TimeUnit.SECONDS)
                .build()

    init {
        mOkHttpClient = okHttpClient
    }

    //post请求主要用于业务接口及部分负责的统计接口
    fun post(url: String, content: String, iProtocolListener: IProtocolListener<String>) {
        post(url, content, true, iProtocolListener)
    }

    fun post(url: String, content: String, userHeader: Boolean, iProtocolListener: IProtocolListener<String>) {
        Logger.v("HttpClientInst", "请求URL： " + url)
        Logger.v("HttpClientInst", "请求参数： " + content)
        val formBody = FormBody.Builder().add(KEY_FORM, content).build()
        val request = Request.Builder().url(url).post(formBody).build()
        request(request, userHeader, iProtocolListener)
    }


    fun uploadImage(url: String, content: String, uploadFile: File, iProtocolListener: IProtocolListener<String>) {
        try {
            val builder = MultipartBody.Builder()
            builder.setType(MultipartBody.FORM)
            builder.addFormDataPart("image", uploadFile.absolutePath, RequestBody.create(MEDIA_TYPE_OCTET, uploadFile))
            builder.addFormDataPart(KEY_FORM, content)
            val requestBody = builder.build()
            val progressRequestBody = ProgressRequestBody(requestBody, null)
            val request = Request.Builder().url(url).post(progressRequestBody).build()
            request(request, true, iProtocolListener)
        } catch (e: Exception) {
            e.printStackTrace()
            AsyncRun.run {
                iProtocolListener?.onFailure(ResponseBean.setResult("发表失败"))
            }
        }

    }

    fun uploadVideo(url: String, content: String, uploadFile: File, start: Long, total: Int, iProtocolListener: IProtocolListener<String>) {
        try {
            val builder = MultipartBody.Builder()
            builder.setType(MultipartBody.FORM)
            val bytes = ByteArray(total)
            val randomAccessFile = RandomAccessFile(uploadFile, "rwd")
            randomAccessFile.seek(start)
            val len = randomAccessFile.read(bytes, 0, total)
            val baos = ByteArrayOutputStream()
            baos.write(bytes, 0, len)
            builder.addFormDataPart("video", uploadFile.absolutePath, RequestBody.create(MEDIA_TYPE_OCTET, baos.toByteArray()))
            builder.addFormDataPart(KEY_FORM, content)
            val requestBody = builder.build()
            val progressRequestBody = ProgressRequestBody(requestBody, null)
            val request = Request.Builder().url(url).post(progressRequestBody).build()
            request(request, true, iProtocolListener)
            baos.close()
        } catch (e: Exception) {
            e.printStackTrace()
            iProtocolListener.onFailure(ResponseBean.setResult("发表失败"))
        }

    }


    private fun request(request: Request, useHeader: Boolean, iProtocolListener: IProtocolListener<String>) {
        val call: Call
        if (useHeader) {
            call = mOkHttpClient!!.newCall(request)
        } else {
            call = OkHttpClient.Builder()
                    .readTimeout(READ_TIMEOUT.toLong(), TimeUnit.SECONDS)//设置读取超时时间
                    .writeTimeout(WRITE_TIMEOUT.toLong(), TimeUnit.SECONDS)//设置写的超时时间
                    .connectTimeout(CONNECT_TIMEOUT.toLong(), TimeUnit.SECONDS)
                    .build()
                    .newCall(request)
        }
        call.enqueue(object : Callback {
            override fun onFailure(call: Call, e: IOException) {
                e.printStackTrace()
                AsyncRun.run {
                    iProtocolListener.onFailure(ResponseBean.setResult("网络请求错误"))
                }
            }

            override fun onResponse(call: Call, response: Response) {
                try {
                    //				Logger.i(response.message());
                    val str = String(response.body().bytes())
                    Logger.v("HttpClientInst", "响应参数： " + str)
                    AsyncRun.run {
                        iProtocolListener.onSuccess(ResponseBean.setResult(str))
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                    AsyncRun.run {
                        iProtocolListener.onSuccess(ResponseBean.setResult("网络请求错误"))
                    }
                }

            }
        })
    }

    fun setOnAddHeaderAction(action: OnAddHeaderAction) {
        this.mAction = action
    }

    interface OnAddHeaderAction {
        fun addHeaderMap(builder: Request.Builder)
    }

    companion object {

        private val KEY_FORM = "data"
        private val MEDIA_TYPE_OCTET = MediaType.parse("application/octet-stream")
        private var mOkHttpClient: OkHttpClient? = null
        private var mInst: HttpClientInst? = null

        val inst: HttpClientInst
            get() {
                synchronized(HttpClientInst::class.java) {
                    if (mInst == null) {
                        mInst = HttpClientInst()
                    }
                }
                return mInst!!
            }

        fun removeCookie() {

        }

        val CONNECT_TIMEOUT = 15
        val READ_TIMEOUT = 20
        val WRITE_TIMEOUT = 15
    }

}
