package com.cc.tools.network

import android.content.Context
import android.os.Handler
import android.text.TextUtils
import com.cc.tools.application.BaseApplication
import com.cc.tools.observer.Observers
import com.cc.tools.utils.ActivityCacheUtils
import com.cc.tools.utils.JSONUtils
import com.cc.tools.utils.MD5Util
import com.cc.tools.utils.ViewUtils
import com.cc.tools.utils.start.HomeModuleUtils
import com.cc.tools.utils.user.UserManager
import org.json.JSONObject
import java.util.*

abstract class ProtocolBase<T> : IProtocolListener<T> {

    var mActionName = ""
    var mToken = ""
    var mCancel = false
    var mContext: Context? = null
    var mListener: IProtocolListener<T>? = null
    var mHandler = Handler()

    protected val baseUrl: String
        get() = HttpConstants.buS_API

    protected val isFinishing: Boolean
        get() = if (Observers.checkNull(mContext)) {
            ViewUtils.ins.isActivityFinish(mContext!!)
        } else true

    fun generateParams(): JSONObject {
        val res = JSONObject()
        val temp = JSONObject()
        try {
            val kvMap = TreeMap<String, Any>()
            setupKeyValues(kvMap)
            kvMap.put("apiKey", HttpConstants.API_KEY)
            val iterator = kvMap.entries.iterator()
            val builder = StringBuilder()
            while (iterator.hasNext()) {
                val entry = iterator.next()
                val key = entry.key
                val value = entry.value
                temp.put(key, value)
                if (value !is String || !TextUtils.isEmpty(value.toString())) {
                    builder.append(entry.key)
                    builder.append("=")
                    builder.append(entry.value)
                    builder.append("&")
                }
            }
            builder.append(mActionName)
            builder.append("&")
            builder.append(HttpConstants.API_SECRET)
            mToken = MD5Util.string2MD5(builder.toString())
            res.put(mActionName, temp)
        } catch (e: Exception) {
        }

        return res
    }

    fun postRequest() {
        if (isFinishing) return
        onStart()
        val paramObj = generateParams()
        val sign = mToken

        val realUrl = String.format(baseUrl, mActionName, sign)

        HttpClientInst.inst.post(realUrl, paramObj.toString(), useHeader(), object : IProtocolListener<String> {
            override fun onStart() {

            }

            override fun onSuccess(bean: ResponseBean<String>) {
                if (isFinishing) return
                if (mCancel) return
                try {
                    readAndAnalysisData(bean.results.toString())
                } catch (e: Exception) {
                    e.printStackTrace()
                    onFailure(ResponseBean.setResult("数据解析出错"))
                }

            }

            override fun onFailure(bean: ResponseBean<String>) {
                if (isFinishing) return
                if (mCancel) return
            }

            override fun onFinish(bean: ResponseBean<String>) {

            }
        })
    }

    @Throws(Exception::class)
    protected fun readAndAnalysisData(str: String?) {
        val responseObj = JSONObject(str)
        var bean: ResponseBean<*> = JSONUtils.parseObject(responseObj.getString(mActionName), ResponseBean::class.java)!!
        if (bean == null) {
            var bean = ResponseBean<T>()
            bean.code = -1
            bean.msg = "数据解析出错"
            onFailure(bean)
        } else {
            when {
                checkLogin(bean.code) -> {
                    bean.msg = "用户未登录，请重新登录！"
                    if (UserManager.inst.isLogin) {
                        UserManager.inst.logout()
                        ActivityCacheUtils.inst.finish()
                        HomeModuleUtils.startToLogin(BaseApplication.mApplication, bean.msg)
                    }
                    var resultBean = ResponseBean<T>()
                    resultBean.code = bean.code
                    resultBean.msg = bean.msg
                    resultBean.curPage = bean.curPage
                    resultBean.count = bean.count
                    resultBean.pageSize = bean.pageSize
                    resultBean.totalSize = bean.totalSize
                    onFailure(resultBean)
                }
                checkCode(bean.code) -> {
                    var resultBean = ResponseBean<T>()
                    resultBean.code = bean.code
                    resultBean.msg = bean.msg
                    resultBean.curPage = bean.curPage
                    resultBean.count = bean.count
                    resultBean.pageSize = bean.pageSize
                    resultBean.totalSize = bean.totalSize
                    try{
                        resultBean!!.results = parseResult(bean.results!!.toString())
                    }catch (e : Exception){
                        e.printStackTrace()
                    }
                    onSuccess(resultBean)
                }
                else -> {
                    var resultBean = ResponseBean<T>()
                    resultBean.code = bean.code
                    resultBean.msg = bean.msg
                    resultBean.curPage = bean.curPage
                    resultBean.count = bean.count
                    resultBean.pageSize = bean.pageSize
                    resultBean.totalSize = bean.totalSize
                    onFailure(resultBean)
                }
            }
        }
    }

    protected fun checkLogin(code: Int): Boolean {
        return code == 1010
    }

    protected fun checkCode(code: Int): Boolean {
        return code == 0
    }

    override fun onStart() {
        mListener?.onStart()
    }

    override fun onSuccess(bean: ResponseBean<T>) {
        mHandler.post(Runnable {
            mListener?.onSuccess(bean)
            onFinish(bean)
        })
    }

    override fun onFailure(bean: ResponseBean<T>) {
        mHandler.post(Runnable {
            mListener?.onFailure(bean)
            onFinish(bean)
        })
    }

    override fun onFinish(bean: ResponseBean<T>) {
        mListener?.onFinish(bean)
    }

    fun setCancel(cancel: Boolean) {
        mCancel = cancel
    }

    abstract fun parseResult(str: String): T

    abstract fun setupKeyValues(kvMap: TreeMap<String, Any>)

    fun useHeader(): Boolean {
        return true
    }

    companion object {
        protected val TAG = ProtocolBase::class.java.simpleName
    }

}
