package com.cc.tools.network.download

import android.app.Application
import android.content.SharedPreferences
import android.os.Handler
import android.os.Looper
import android.preference.PreferenceManager
import android.text.TextUtils
import com.cc.tools.application.BaseApplication
import com.cc.tools.observer.Observers
import com.cc.tools.utils.*
import java.io.File
import java.io.RandomAccessFile
import java.net.HttpURLConnection
import java.net.URL

class DownloadManager private constructor() : Observers<IDownloadProgress>(), IDownloadProgress {
    val mDownloadUrlList = mutableListOf<String>()
    var mPreferences: SharedPreferences? = null
    var mApplication: Application? = null

    fun initApplication(application: BaseApplication) {
        mApplication = application
        mPreferences = PreferenceManager.getDefaultSharedPreferences(application)
        for ((key, value) in mPreferences!!.all) {
            try {
                var str = mPreferences!!.getString(key, "")
                var bean = JSONUtils.parseObject(str, DownloadFileBean::class.java)
                when (bean!!.state) {
                    DOWNLOAD_ING -> {
                        bean.state = DOWNLOAD_PAUSE
                        writeDownloadFileBean(bean)
                    }
                }
            } catch (e: Exception) {

            }
        }
    }

    override fun onDownloadIde(bean: DownloadFileBean) {
    }

    override fun onDownloadProgress(bean: DownloadFileBean) {
        invokeMethod("onDownloadProgress", mutableListOf(bean))
    }

    override fun onDownloadEnd(bean: DownloadFileBean) {
        invokeMethod("onDownloadEnd", mutableListOf(bean))
    }

    override fun onDownloadPaused(bean: DownloadFileBean) {
        invokeMethod("onDownloadPaused", mutableListOf(bean))
    }

    fun addDownload(title: String, url: String, totalSize: Long, pkgName: String, savePath: String) {
        addDownload(title, url, totalSize, pkgName, savePath, true)
    }

    fun addDownload(title: String, url: String, totalSize: Long, pkgName: String, savePath: String, boolean: Boolean) {
        var str = mPreferences!!.getString(url, "")
        var bean: DownloadFileBean?
        if (TextUtils.isEmpty(str)) {
            bean = DownloadFileBean()
            bean.totalBytes = totalSize
            bean.title = title
            bean.url = url
            bean.pkgName = pkgName
            bean.savePath = savePath
            bean.state = DOWNLOAD_ING
            bean.currentBytes = 0
            mPreferences!!.edit().putString(url, JSONUtils.toJSONString(bean))
        } else {
            bean = JSONUtils.parseObject(str, DownloadFileBean::class.java)
            bean!!.savePath = savePath
        }
        bean!!.state = DOWNLOAD_ING
        Thread(Runnable {
            try {
                val httpUrl = URL(bean!!.url)
                val http = httpUrl.openConnection() as HttpURLConnection
                HttpURLConnection.setFollowRedirects(true)
                http.connectTimeout = 5 * 1000
                http.readTimeout = 12 * 1000
                http.requestMethod = "GET"
                http.setRequestProperty("Accept", "image/gif, image/jpeg, image/pjpeg, image/pjpeg, application/x-shockwave-flash, application/xaml+xml, application/vnd.ms-xpsdocument, application/x-ms-xbap, application/x-ms-application, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, */*")     //设置客户端可以接受的返回数据类型
                http.setRequestProperty("Accept-Language", "zh-CN")     //设置客户端使用的语言问中文
                http.setRequestProperty("Referer", bean!!.url)     //设置请求的来源，便于对访问来源进行统计
                http.setRequestProperty("Charset", "UTF-8")     //设置通信编码为UTF-8
                http.setRequestProperty("Range", "bytes=" + File(bean!!.savePath).length() + "-")
                http.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)")     //客户端用户代理
                http.setRequestProperty("Connection", "Keep-Alive")     //使用长连接
                var inStream = http.inputStream
                writeDownloadFileBean(bean!!)
                val buffer = ByteArray(1024 * 1024)     //设置本地数据缓存的大小为16k
                var offset = 0
                var file = File(bean!!.savePath)
                file.parentFile.mkdirs()
                var raFile = RandomAccessFile(file, "rw")
                raFile.seek(bean!!.currentBytes)
                bean!!.state = DownloadManager.DOWNLOAD_ING
                writeDownloadFileBean(bean!!)
                var currentTime = System.currentTimeMillis()
                while (true) {
                    offset = inStream!!.read(buffer)
                    if (offset == -1) break
                    raFile.write(buffer, 0, offset)
                    bean!!.currentBytes = bean!!.currentBytes + offset
                    val now = System.currentTimeMillis()
                    if (now - currentTime >= 750) {
                        writeDownloadFileBean(bean!!)
                        currentTime = now
                    }
                }
                bean!!.state = DOWNLOAD_ED
                writeDownloadFileBean(bean!!)
                if (boolean) {
                    HandlerUtils.post(Handler(Looper.getMainLooper()), Runnable {
                        ToastUtils.showLongToast(mApplication!!, bean!!.title + "下载完成，请使用")
                    })
                } else {
                    AppManager.installApp(BaseApplication.mApplication, bean!!.savePath)
                }
                raFile.close()
                inStream!!.close()
            } catch (e: Exception) {
                bean!!.state = DOWNLOAD_PAUSE
                writeDownloadFileBean(bean!!)
                File(bean!!.savePath).delete()
                e.printStackTrace()
            }
        }).start()
    }

    fun writeDownloadFileBean(bean: DownloadFileBean) {
        when (bean.state) {
            DOWNLOAD_ING -> onDownloadProgress(bean)
            DOWNLOAD_PAUSE -> onDownloadPaused(bean)
            DOWNLOAD_ED -> onDownloadEnd(bean)
        }
        mPreferences!!.edit().putString(bean.url, JSONUtils.toJSONString(bean)).apply()
    }

    fun getDownloadFileBean(url: String): DownloadFileBean {
        var str = mPreferences!!.getString(url, "")
        Logger.i(str)
        if (TextUtils.isEmpty(str)) {
            var bean = DownloadFileBean()
            bean.url = url
            return bean
        } else {
            return JSONUtils.parseObject(str, DownloadFileBean::class.java)!!
        }
    }

    fun deleteDownloadInfo(url: String) {
        mPreferences!!.edit().remove(url).apply()
    }

    companion object {

        val DOWNLOAD_IDE = -1
        val DOWNLOAD_ING = 1
        val DOWNLOAD_ED = 2
        val DOWNLOAD_PAUSE = 3

        private var mInst: DownloadManager? = null

        val inst: DownloadManager
            get() {
                synchronized(DownloadManager::class.java) {
                    if (mInst == null) {
                        mInst = DownloadManager()

                    }
                }
                return mInst!!
            }

    }
}
