package com.cc.tools.network.download

import com.cc.tools.model.common.IUnZipModel
import com.cc.tools.model.common.IZipModel
import com.cc.tools.utils.CacheUtils
import java.io.*
import java.util.zip.ZipEntry
import java.util.zip.ZipFile
import java.util.zip.ZipOutputStream

/**
 * cpk安装
 *
 * @author zhangbp
 */
object ZipUtils {

    val BUFF_SIZE = 1024 * 1024

    fun unZip(path: String, iUnZipModel: IUnZipModel) {
        Thread(Runnable {
            var tempFile = CacheUtils.getDQMR()
            CacheUtils.delDir(tempFile)
            var bis: BufferedInputStream? = null
            var out: OutputStream? = null
            var bos: BufferedOutputStream? = null
            var zf: ZipFile? = null
            try {
                val mFile = File(path)
                zf = ZipFile(mFile)
                var completeSize: Long = getCpkDataCount(zf)
                var progress: Long = 0
                val entries = zf.entries()
                while (entries.hasMoreElements()) {
                    val entry = entries.nextElement() as ZipEntry
                    if (entry.isDirectory) {
                        File(tempFile, entry.name).mkdirs()
                        continue
                    }
                    bis = BufferedInputStream(zf.getInputStream(entry))
                    val desFile = File(tempFile, entry.name)
                    if (!desFile.exists()) {
                        val fileParentDir = desFile.parentFile
                        if (!fileParentDir.exists()) {
                            fileParentDir.mkdirs()
                        }
                        desFile.createNewFile()
                    }
                    out = FileOutputStream(desFile)
                    bos = BufferedOutputStream(out, BUFF_SIZE)

                    val buffer = ByteArray(BUFF_SIZE)
                    var realLength: Int
                    while (true) {
                        realLength = bis.read(buffer)
                        if (realLength == -1) break
                        bos.write(buffer, 0, realLength)
                        progress += realLength
                        iUnZipModel?.onProgress(progress, completeSize)
                    }
                    bos.flush()
                    bos.close()
                    bis.close()
                }
                zf.close()
                iUnZipModel?.onComplete()
            } catch (e: Exception) {
                e.printStackTrace()
                iUnZipModel?.onError()
            } finally {
                if (bos != null) {
                    try {
                        bos.close()
                    } catch (e: IOException) {
                        e.printStackTrace()
                    }

                    bos = null
                }
                if (out != null) {
                    try {
                        out.close()
                    } catch (e: IOException) {
                        e.printStackTrace()
                    }

                    out = null
                }
                if (bis != null) {
                    try {
                        bis.close()
                    } catch (e: IOException) {
                        e.printStackTrace()
                    }

                    bis = null
                }

                if (zf != null) {
                    try {
                        zf.close()
                    } catch (e: IOException) {
                        e.printStackTrace()
                    }

                    zf = null
                }
            }
        }).start()
    }

    fun getCpkDataCount(cpk: ZipFile): Long {
        var fileTotalCount: Long = 0
        val emu = cpk.entries()
        while (emu.hasMoreElements()) {
            val entry = emu.nextElement() as ZipEntry
            fileTotalCount += entry.size
        }
        return fileTotalCount
    }

    fun zip(zipFile: File, iZipModel: IZipModel) {
        Thread(Runnable {
            try {
                iZipModel.onZipStart()
                var tempFile = CacheUtils.getDQMR()
                var fos = FileOutputStream(zipFile)
                var zipOut = ZipOutputStream(fos)
                compress(zipOut, tempFile, tempFile, "", iZipModel)
                iZipModel.onZipComplete()
                zipOut.close()
            } catch (e: Exception) {
                iZipModel.onZipError()
            }
        }).start()
    }

    fun compress(zipOut: ZipOutputStream, sourceFile: File, tempeFile: File, base: String, iZipModel: IZipModel) {
        if (sourceFile.isDirectory) {
            var flist = sourceFile.listFiles()
            if (flist == null || flist.isEmpty()) {
                var zipEntry = ZipEntry(base + "/")
                zipOut.putNextEntry(zipEntry)
                zipOut.closeEntry()
                iZipModel.onZipProgress(zipEntry.name)
            } else {
                if (sourceFile == tempeFile) {
                    flist
                            .filter { it.name == "files" || it.name == "cache" }
                            .forEach { compress(zipOut, it, tempeFile, base + "/" + it.name, iZipModel) }
                } else {
                    flist.forEach {
                        compress(zipOut, it, tempeFile, base + "/" + it.name, iZipModel)
                    }
                }
            }
        } else {
            var zipEntry = ZipEntry(base)
            zipOut.putNextEntry(zipEntry)
            iZipModel.onZipProgress(zipEntry.name)
            println(sourceFile.length())
            val fis = FileInputStream(sourceFile)
            val bis = BufferedInputStream(fis)
            val buffer = ByteArray(BUFF_SIZE)
            while (true) {
                var tag = bis.read(buffer)
                if (tag == -1) break
                zipOut.write(buffer, 0, tag)
            }
            zipOut.closeEntry()
            bis.close()
            fis.close()
        }
    }
}
