package com.cc.tools.observer

import java.lang.ref.Reference
import java.util.*

open class Observers<T : Any> {

    protected var mListeners: MutableList<T> = mutableListOf()
    protected var mActionMap: MutableMap<Any, MutableList<T>> = mutableMapOf()

    val isEmpty: Boolean
        get() = mListeners.isEmpty()

    @Synchronized
    open fun checkExit(listeners: MutableList<T>, l: T): Boolean {
        val iterator = listeners.iterator()
        while (iterator.hasNext()) {
            val wr = iterator.next()
            if (wr == null) {
                iterator.remove()
            } else if (l == wr) {
                return true
            }
        }
        return false
    }

    @Synchronized
    open fun addListener(t: T) {
        if (checkExit(mListeners, t)) return
        mListeners.add(t)
    }

    @Synchronized
    open fun removeListener(t: T) {
        removeListener(mListeners, t)
    }

    @Synchronized
    open fun removeListener(list: MutableList<T>, t: T) {
        try {
            val iterator = list.iterator()
            while (iterator.hasNext()) {
                val wr = iterator.next()
                if (wr == null) {
                    iterator.remove()
                } else if (t == wr) {
                    iterator.remove()
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    @Synchronized
    open fun addListener(listeners: MutableList<T>, t: T) {
        if (checkExit(listeners, t)) return
        listeners.add(t)
    }


    @Synchronized
    fun addListener(o: Any, t: T) {
        var list: MutableList<T>? = mActionMap[o]
        if (list == null) {
            list = ArrayList()
            mActionMap.put(o, list)
        }
        addListener(list, t)
    }

    @Synchronized
    fun removeListener(o: Any, t: T) {
        val list = mActionMap[o]
        if (list != null) {
            removeListener(list, t)
            if (list.isEmpty()) {
                mActionMap.remove(o)
            }
        }
    }

    fun clear() {
        mListeners.clear()
        mActionMap.clear()
    }

    fun clear(o: Any) {
        val list = mActionMap.remove(o)
        list?.clear()
    }

    fun invokeMethod(method: String, values: MutableList<Any>) {
        val iterator = mListeners.iterator()
        while (iterator.hasNext()) {
            val wr = iterator.next()
            if (wr == null) {
                iterator.remove()
            } else {
                try {
                    val parameterTypes = mutableListOf<Class<*>>()
                    val args = mutableListOf<Any>()
                    for (i in 0 until values.size) {
                        val o = values.get(i)
                        val aClass = o.javaClass
                        if (aClass == String::class.java) {
                            parameterTypes.add(String::class.java)
                        } else if (aClass == Int::class.java) {
                            parameterTypes.add(Int::class.java)
                        } else if (aClass == Boolean::class.java) {
                            parameterTypes.add(Boolean::class.java)
                        } else {
                            parameterTypes.add(aClass)
                        }
                        args.add(o)
                    }
                    val cls = wr.javaClass
                    val method = cls.getMethod(method, *parameterTypes.toTypedArray())
                    method.invoke(wr, *args.toTypedArray())
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
        }
    }

    companion object {

        fun checkNull(t: Reference<*>?): Boolean {
            return t?.get() != null
        }

        fun checkNull(t: Any?): Boolean {
            return t != null
        }
    }

}
