package com.cc.tools.presenter

import com.cc.tools.bean.CacheBean
import com.cc.tools.bean.EntityCdnTokenBean
import com.cc.tools.bean.user.UserInfoBean
import com.cc.tools.model.IMainHeaderModel
import com.cc.tools.model.common.IZipModel
import com.cc.tools.network.IProtocolListener
import com.cc.tools.network.ResponseBean
import com.cc.tools.network.download.ZipUtils
import com.cc.tools.network.protocol.cache.ProtocolArchiveAdd
import com.cc.tools.network.protocol.cache.ProtocolArchiveUpdate
import com.cc.tools.network.protocol.cache.ProtocolArchiveUserList
import com.cc.tools.network.protocol.cache.ProtocolCdnTokenFile
import com.cc.tools.network.protocol.login.ProtocolUserInfo
import com.cc.tools.utils.HandlerUtils
import com.cc.tools.utils.MD5Util
import com.cc.tools.utils.ToastUtils
import com.qiniu.android.http.ResponseInfo
import com.qiniu.android.storage.*
import com.qiniu.android.storage.persistent.FileRecorder
import java.io.File


class IMainHeaderPresenter : IPresenterImp<IMainHeaderModel> {

    private val CACHE_RECORD_PATH = "QI_NIU"

    constructor() {}

    override fun initData() {

    }

    fun getUserInfo() {
        var protocol = ProtocolUserInfo()
        protocol.mContext = mCtx
        protocol.mListener = object : IProtocolListener<UserInfoBean> {
            override fun onStart() {
            }

            override fun onSuccess(bean: ResponseBean<UserInfoBean>) {
                mIModelImp?.setUserInfo(bean.results!!)
            }

            override fun onFailure(bean: ResponseBean<UserInfoBean>) {
                ToastUtils.showLongToast(mCtx!!, bean.msg)
            }

            override fun onFinish(bean: ResponseBean<UserInfoBean>) {
            }
        }
        protocol.postRequest()
    }

    fun getUserCacheList() {
        var protocolArchive = ProtocolArchiveUserList()
        protocolArchive.page = 1
        protocolArchive.page_size = 5
        protocolArchive.mContext = mCtx
        protocolArchive.mListener = object : IProtocolListener<MutableList<CacheBean>> {
            override fun onStart() {
            }

            override fun onSuccess(bean: ResponseBean<MutableList<CacheBean>>) {
                if (bean.results == null) {
                    bean.results = mutableListOf()
                }
                mIModelImp?.onFirstRequestSuccess(bean)
            }

            override fun onFailure(bean: ResponseBean<MutableList<CacheBean>>) {
                mIModelImp?.onFirstRequestFail(bean)
            }

            override fun onFinish(bean: ResponseBean<MutableList<CacheBean>>) {
            }
        }
        protocolArchive.postRequest()
    }

    fun zipCache(name: String, id: Int) {
        var file = File(mCtx!!.getExternalFilesDir(""), name)
        if (file.exists()) {
            file.delete()
        }
        ZipUtils.zip(file, object : IZipModel {

            override fun onZipStart() {
                mIModelImp?.onZipStart()
            }

            override fun onZipError() {
                mIModelImp?.onZipError()
            }

            override fun onZipComplete() {
                mIModelImp?.onZipComplete()
                HandlerUtils.post(mHandler, Runnable {
                    upload(file, id)
                })
            }

            override fun onZipProgress(name: String) {
                println(name)
                mIModelImp?.onZipProgress(name)
            }
        })
    }

    fun upload(file: File, id: Int) {
        var protocol = ProtocolCdnTokenFile()
        protocol.mContext = mCtx
        mIModelImp?.IUploadStart()
        protocol.mListener = object : IProtocolListener<EntityCdnTokenBean> {
            override fun onStart() {
            }

            override fun onSuccess(bean: ResponseBean<EntityCdnTokenBean>) {
                var md5 = MD5Util.getFileMD5(file)
                val uploadOptions = UploadOptions(null, null, false,
                        UpProgressHandler { key, percent -> mIModelImp?.IUploadProgress((percent * 100).toLong(), 100) },
                        UpCancellationSignal {
                            false
                        })
                val recorder = FileRecorder(File(mCtx!!.filesDir, CACHE_RECORD_PATH).absolutePath)
                val keyGen = KeyGenerator { key, file -> md5!! }
                val config = Configuration.Builder().recorder(recorder, keyGen).build()
                val uploadManager = UploadManager(config)
                val token = bean.results!!.uploadToken
                var keyPrefix = bean.results!!.keyPrefix
                var key = keyPrefix + md5
                uploadManager.put(file, key!!, token, { key, info, response ->
                    if (info == null) {
                        mIModelImp?.IUploadError()
                    } else if (ResponseInfo.ZeroSizeFile == info.statusCode) {
                        mIModelImp?.IUploadError()
                    } else if (ResponseInfo.InvalidToken == info.statusCode) {
                        mIModelImp?.IUploadError()
                    } else if (ResponseInfo.InvalidArgument == info.statusCode) {
                        mIModelImp?.IUploadError()
                    } else if (ResponseInfo.InvalidFile == info.statusCode) {
                        mIModelImp?.IUploadError()
                    } else if (ResponseInfo.Cancelled == info.statusCode) {
                        mIModelImp?.IUploadError()
                    } else if (info.isNetworkBroken) {
                        mIModelImp?.IUploadError()
                    } else if (info.isServerError) {
                        mIModelImp?.IUploadError()
                    } else if (info.isNotQiniu) {
                        mIModelImp?.IUploadError()
                    } else if (info.isOK) {
                        mIModelImp?.IUploadSuccess()
                        if (keyPrefix.startsWith("media/")) {
                            keyPrefix = keyPrefix.substring(6)
                        }
                        var archiveUrl = keyPrefix + md5
                        if (id >= 0) {
                            updateUserCache(id, file.name, archiveUrl, file.length())
                        } else {
                            addUserCache(file.name, archiveUrl, file.length())
                        }
                    }
                }, uploadOptions)
            }

            override fun onFailure(bean: ResponseBean<EntityCdnTokenBean>) {
                ToastUtils.showLongToast(mCtx!!, "上传失败，请检查网络")
            }

            override fun onFinish(bean: ResponseBean<EntityCdnTokenBean>) {
            }
        }
        protocol.postRequest()
    }

    fun updateUserCache(id: Int, archiveName: String, archiveUrl: String, archiveSize: Long) {
        var protocol = ProtocolArchiveUpdate()
        protocol.id = id
        protocol.archiveName = archiveName
        protocol.archiveUrl = archiveUrl
        protocol.archiveSize = archiveSize
        protocol.mContext = mCtx
        protocol.mListener = object : IProtocolListener<String> {
            override fun onStart() {
            }

            override fun onSuccess(bean: ResponseBean<String>) {
                getUserCacheList()
                ToastUtils.showLongToast(mCtx!!, "用户存档更新成功")
            }

            override fun onFailure(bean: ResponseBean<String>) {
                ToastUtils.showLongToast(mCtx!!, "用户数据提交失败")
            }

            override fun onFinish(bean: ResponseBean<String>) {
            }
        }
        protocol.postRequest()
    }

    fun addUserCache(archiveName: String, archiveUrl: String, archiveSize: Long) {
        var protocol = ProtocolArchiveAdd()
        protocol.archiveName = archiveName
        protocol.archiveUrl = archiveUrl
        protocol.archiveSize = archiveSize
        protocol.mContext = mCtx
        protocol.mListener = object : IProtocolListener<String> {
            override fun onStart() {
            }

            override fun onSuccess(bean: ResponseBean<String>) {
                getUserCacheList()
                ToastUtils.showLongToast(mCtx!!, "用户存档添加成功")
            }

            override fun onFailure(bean: ResponseBean<String>) {
                ToastUtils.showLongToast(mCtx!!, "用户数据提交失败")
            }

            override fun onFinish(bean: ResponseBean<String>) {
            }
        }
        protocol.postRequest()
    }
}