package com.cc.tools.utils

import android.app.Activity
import android.app.Application
import android.content.Context
import android.graphics.Bitmap
import android.graphics.drawable.Drawable
import android.os.Looper
import android.support.v4.app.Fragment
import android.support.v4.app.FragmentActivity
import android.text.TextUtils
import android.widget.ImageView
import com.bumptech.glide.Glide
import com.bumptech.glide.Priority
import com.bumptech.glide.RequestBuilder
import com.bumptech.glide.RequestManager
import com.bumptech.glide.load.Transformation
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.bumptech.glide.load.engine.cache.ExternalCacheDiskCacheFactory
import com.bumptech.glide.load.resource.bitmap.CircleCrop
import com.bumptech.glide.request.RequestOptions
import com.cc.tools.R
import java.io.File


class GlideUtils private constructor() {

//    private fun loadCircleBitmap(o: Any, url: String, defaultRes: Int, imageView: ImageView?, cache: Boolean) {
//        if (imageView == null) {
//            return
//        }
//        getRequestManager(o).load(url.trim { it <= ' ' }).skipMemoryCache(!cache).diskCacheStrategy(if (cache) DiskCacheStrategy.SOURCE else DiskCacheStrategy.NONE).placeholder(defaultRes).error(defaultRes).dontAnimate().into(imageView)
//    }

    private lateinit var mApplication: Application

    init {

    }

    fun initApplication(application: Application) {
        mApplication = application
    }

    fun loadBitmap(o: Any, url: String, imageView: ImageView) {
        loadBitmap(o, url, R.color.color_bg, imageView)
    }
    fun loadBitmap(o: Any, url: String, defaultRes: Int, imageView: ImageView) {
        loadBitmap(o, url, R.color.color_bg, imageView, null)
    }

    fun loadBitmap(o: Any, url: String, defaultRes: Int, imageView: ImageView, transformation: Transformation<Bitmap>?) {
        var url = url
        if (TextUtils.isEmpty(url)) {
            url = ""
        }
        buildDrawableRequestBuilder(o, url, defaultRes, transformation).into(imageView)
    }

    fun loadUserIcon(o: Any, url: String, imageView: ImageView) {
        loadBitmap(o, url, R.drawable.tools_user_default, imageView, CircleCrop())
    }

    fun buildDrawableRequestBuilder(o: Any, url: String, defaultRes: Int, transformation: Transformation<Bitmap>?): RequestBuilder<Drawable> {
        var url = url
        if (TextUtils.isEmpty(url)) {
            url = ""
        }
        var integer: Int? = 0
        try {
            integer = Integer.parseInt(url)
        } catch (e: Exception) {

        }
        val requestManager = getRequestManager(o)
        val drawableTypeRequest: RequestBuilder<Drawable>
        val options = RequestOptions()
                .centerCrop()
                .placeholder(defaultRes)
                .error(defaultRes)
                .priority(Priority.HIGH)
                .diskCacheStrategy(DiskCacheStrategy.RESOURCE);
        if (transformation != null) {
            options.transform(transformation)
        }
        if (integer == 0) {
            drawableTypeRequest = requestManager.load(url.trim { it <= ' ' }).apply(options)
        } else {
            drawableTypeRequest = requestManager.load(integer).apply(options)
        }

        return drawableTypeRequest
    }

    fun pauseRequests(o: Any) {
        getRequestManager(o).pauseRequests()
    }

    fun resumeRequests(o: Any) {
        getRequestManager(o).resumeRequests()
    }

    private fun getContent(o: Any): Context {
        return if (o is FragmentActivity) {
            o.applicationContext
        } else if (o is Activity) {
            o.applicationContext
        } else if (o is Fragment) {
            o.context
        } else if (o is android.app.Fragment) {
            o.activity
        } else {
            o as Context
        }
    }

    private fun getRequestManager(o: Any): RequestManager {
        val requestManager: RequestManager
        if (o is FragmentActivity) {
            requestManager = Glide.with(o)
        } else if (o is Activity) {
            requestManager = Glide.with(o)
        } else if (o is Fragment) {
            requestManager = Glide.with(o)
        } else if (o is android.app.Fragment) {
            requestManager = Glide.with(o)
        } else {
            requestManager = Glide.with(o as Context)
        }
        return requestManager
    }

    /**
     * 清除图片磁盘缓存
     */
    fun clearImageDiskCache(context: Context, onClearListener: OnClearListener) {
        try {
            if (Looper.myLooper() == Looper.getMainLooper()) {
                Thread(Runnable {
                    Glide.get(context).clearDiskCache()
                    val ImageExternalCatchDir = context.externalCacheDir!!.toString() + ExternalCacheDiskCacheFactory.DEFAULT_DISK_CACHE_DIR
                    deleteFolderFile(ImageExternalCatchDir, true)
                    onClearListener.onClearSuccess()
                }).start()
            } else {
                Glide.get(context).clearDiskCache()
                val ImageExternalCatchDir = context.externalCacheDir!!.toString() + ExternalCacheDiskCacheFactory.DEFAULT_DISK_CACHE_DIR
                deleteFolderFile(ImageExternalCatchDir, true)
                onClearListener.onClearSuccess()
            }
        } catch (e: Exception) {
            e.printStackTrace()
            onClearListener.onClearFail()
        }

    }

    /**
     * 清除图片内存缓存
     */
    fun clearImageMemoryCache(context: Context) {
        try {
            if (Looper.myLooper() == Looper.getMainLooper()) { //只能在主线程执行
                Glide.get(context).clearMemory()
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }

    }

    /**
     * 清除图片所有缓存
     */
    fun clearImageAllCache(context: Context, onClearListener: OnClearListener) {
        onClearListener.onClearStart()
        clearImageMemoryCache(context)
        clearImageDiskCache(context, onClearListener)
    }

    /**
     * 删除指定目录下的文件，这里用于缓存的删除
     *
     * @param filePath filePath
     * @param deleteThisPath deleteThisPath
     */
    private fun deleteFolderFile(filePath: String, deleteThisPath: Boolean) {
        if (!TextUtils.isEmpty(filePath)) {
            try {
                val file = File(filePath)
                if (file.isDirectory) {
                    val files = file.listFiles()
                    for (file1 in files) {
                        deleteFolderFile(file1.absolutePath, true)
                    }
                }
                if (deleteThisPath) {
                    if (!file.isDirectory) {
                        file.delete()
                    } else {
                        if (file.listFiles().size == 0) {
                            file.delete()
                        }
                    }
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }

        }
    }

    /**
     * 获取指定文件夹内所有文件大小的和
     *
     * @param file file
     * @return size
     * @throws Exception
     */
    @Throws(Exception::class)
    private fun getFolderSize(file: File): Long {
        var size: Long = 0
        try {
            val fileList = file.listFiles()
            for (aFileList in fileList) {
                if (aFileList.isDirectory) {
                    size = size + getFolderSize(aFileList)
                } else {
                    size = size + aFileList.length()
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }

        return size
    }

    companion object {

        private var mIns: GlideUtils? = null

        val ins: GlideUtils
            get() {
                synchronized(GlideUtils::class.java) {
                    if (mIns == null) {
                        mIns = GlideUtils()
                    }
                }
                return mIns!!
            }
    }

    interface OnClearListener {

        fun onClearStart()
        fun onClearSuccess()
        fun onClearFail()

    }

}//加载用户图标
