package com.cc.tools.utils

import android.app.Application
import android.bluetooth.BluetoothAdapter
import android.content.Context
import android.content.pm.PackageManager
import android.hardware.SensorManager
import android.location.LocationManager
import android.net.ConnectivityManager
import android.net.wifi.WifiManager
import android.os.Build
import android.os.Environment
import android.os.StatFs
import android.telephony.TelephonyManager
import android.text.TextUtils
import android.util.DisplayMetrics
import android.view.WindowManager
import org.json.JSONObject
import java.io.UnsupportedEncodingException
import java.net.URLEncoder
import java.util.*


class MachineInfoUtil private constructor() {

    var IMEI = "0000000000000000"
    var IMSI = ""
    var MANUFACTURER = ""
    var MODEL = ""
    var DEVICE_TYPE = ""
    var SDK_VERSION = 3
    var SDK_: String = ""
    var DENSITY = 160
    var CONNECT_TYPE = ""
    var ANDROID_ID = ""
    var HARDWARE = "default"
    //ro.build.display.id
    var DISPLAY_ID = "unknown"
    //4.2.2
    var SDK_VERSION_STR = "unknown"
    var BUILD_BOARD = "unknown"
    var RESOLUTION = ""
    var SSH_CLIENT_VERSION_CODE = 0
    var SSH_CLIENT_VERSION_NAME: String = ""
    var CPU_PLATFORM = ""
    private var mContext: Context? = null
    var mIsArmeabi: Boolean = false
    var MAC_ADDRESS: String = ""

    val imei: String
        get() {
            var imei = ""
            try {
                val tm = mContext!!.getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
                imei = tm.deviceId
                if (TextUtils.isEmpty(imei)) {
                    imei = tm.simSerialNumber
                }
            } catch (e: Exception) {

            }

            if (TextUtils.isEmpty(imei)) {
                imei = "0000000000000000"
            }
            return imei
        }

    private val imsi: String
        get() {
            var imsi = ""
            try {
                val tm = mContext!!.getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
                imsi = tm.subscriberId
            } catch (e: Exception) {

            }

            if (TextUtils.isEmpty(imsi)) {
                imsi = ""
            }
            return imsi
        }

    private val phoneType: Int
        get() {
            try {
                val tm = mContext!!.getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
                return tm.phoneType
            } catch (e: Exception) {
                return -1
            }

        }

    private val androiD_ID: String
        get() {
            try {
                return ReflectionUtils.androidID
            } catch (e: Exception) {
                return UUID.randomUUID().toString()
            }

        }

    val resolution: String
        get() {
            val dm = DisplayMetrics()
            val mgr = mContext!!.getSystemService(Context.WINDOW_SERVICE) as WindowManager
            mgr.defaultDisplay.getMetrics(dm)
            return "" + dm.heightPixels + "x" + dm.widthPixels
        }

    val clientHeader: String
        get() {
            val jsonObject = JSONObject()
            try {
                jsonObject.put("channel_name", encode(ChannelManage.getInstance().channel))
                jsonObject.put("imei", encode(IMEI))
                jsonObject.put("version_name", encode(SSH_CLIENT_VERSION_NAME))
                jsonObject.put("platform", encode(CPU_PLATFORM))
                jsonObject.put("wifi_mac", encode(MAC_ADDRESS))
                jsonObject.put("package_name", encode(mContext!!.packageName))
                jsonObject.put("model_name", encode(MODEL))
                jsonObject.put("version_code", SSH_CLIENT_VERSION_CODE)
                jsonObject.put("os_version", encode(SDK_))
                jsonObject.put("manufacturer", encode(Build.MANUFACTURER))
                jsonObject.put("module_name", encode(Build.PRODUCT))
                jsonObject.put("imsi", encode(IMSI))
                jsonObject.put("language", encode(Locale.getDefault().language))
                jsonObject.put("is_mobiledevice", true)
                jsonObject.put("phone_type", phoneType)
                jsonObject.put("resolution", encode(resolution))
                jsonObject.put("have_wifi", NetworkUtils.isWifiActive(mContext!!))

                try {
                    val locationManager = mContext!!.getSystemService(Context.LOCATION_SERVICE) as LocationManager
                    jsonObject.put("have_gps", locationManager != null)
                } catch (e: Exception) {
                }

                jsonObject.put("device_name", encode(deviceName))
                jsonObject.put("have_gravity", isHaveGravity(mContext!!))

                try {
                    val adapter = BluetoothAdapter.getDefaultAdapter()
                    jsonObject.put("have_bt", adapter != null)
                } catch (e: Exception) {
                }

            } catch (e: Exception) {
                e.printStackTrace()
            }

            return jsonObject.toString()
        }

    fun initInfo(application: Application) {
        mContext = application
        IMEI = checkString(imei)
        IMSI = checkString(imsi)
        ANDROID_ID = checkString(androiD_ID)
        val connectType = NetworkUtils.getConnectType(mContext!!)
        if (connectType == ConnectivityManager.TYPE_WIFI) {
            CONNECT_TYPE = "wifi"
        } else {
            CONNECT_TYPE = "2g/3g"
        }
        SSH_CLIENT_VERSION_CODE = getVersionCode(mContext)
        SSH_CLIENT_VERSION_NAME = checkString(getVersionName(mContext))
        CPU_PLATFORM = checkString(android.os.Build.CPU_ABI)
        mIsArmeabi = CPU_PLATFORM.contains(ARM)
        RESOLUTION = checkString(resolution)
        getOtherInfo()

        MAC_ADDRESS = checkString(getMacAddress())
    }

    private fun checkString(str: String?): String {
        return if (TextUtils.isEmpty(str)) "null" else str!!
    }

    private fun getOtherInfo() {
        try {
            DENSITY = mContext!!.resources.displayMetrics.densityDpi
            SDK_VERSION = android.os.Build.VERSION.SDK_INT
            SDK_ = android.os.Build.VERSION.RELEASE//系统版本
            checkString(SDK_)

            MANUFACTURER = checkString(android.os.Build.MANUFACTURER)
            MODEL = checkString(android.os.Build.MODEL)
            DEVICE_TYPE = checkString(android.os.Build.DEVICE)
            HARDWARE = checkString(android.os.Build.HARDWARE)
            DISPLAY_ID = checkString(android.os.Build.DISPLAY)
            SDK_VERSION_STR = checkString(android.os.Build.VERSION.RELEASE)
            BUILD_BOARD = checkString(checkString(android.os.Build.BOARD))
        } catch (e: Exception) {
            e.printStackTrace()
        }

    }

    fun getMacAddress(): String? {
        try {
            val wifiManager = mContext!!.getSystemService(Context.WIFI_SERVICE) as WifiManager
            val info = wifiManager.connectionInfo
            if (info != null) {
                return info.macAddress
            }
        } catch (e: Exception) {
        }

        return null
    }

    private fun encode(s: String): String {
        try {
            return URLEncoder.encode(s, "utf-8")
        } catch (e: UnsupportedEncodingException) {
            return URLEncoder.encode(s)
        }

    }

    companion object {

        private val ARM = "armeabi"

        /**
         * 手机信息管理实例
         */
        private var mInst: MachineInfoUtil? = null
        val instance: MachineInfoUtil
            get() {
                if (null == mInst) {
                    mInst = MachineInfoUtil()
                }
                return mInst!!
            }


        fun externalMemoryAvailable(): Boolean {
            return android.os.Environment.getExternalStorageState() == android.os.Environment.MEDIA_MOUNTED
        }

        /**
         * 获取可用外部存储容量
         *
         * @return
         * @time 2012-2-28 | 下午01:57:40
         */
        val availableExternalMemorySize: Long
            get() {
                if (externalMemoryAvailable()) {
                    val path = Environment.getExternalStorageDirectory()
                    val stat = StatFs(path.path)
                    val blockSize = stat.blockSize.toLong()
                    val availableBlocks = stat.availableBlocks.toLong()
                    return availableBlocks * blockSize
                } else {
                    return -1
                }
            }

        /**
         * 获取可用内部存储容量
         *
         * @return
         * @author xxj
         * @time 2012-2-28 | 下午01:57:59
         */
        val availableInternalMemorySize: Long
            get() {
                val path = Environment.getDataDirectory()
                val stat = StatFs(path.path)
                val blockSize = stat.blockSize.toLong()
                val availableBlocks = stat.availableBlocks.toLong()
                return availableBlocks * blockSize
            }

        /**
         * 获取当前软件版本号
         *
         * @param context
         * @return
         * @author xxj
         * @time 2012-2-28 | 下午02:13:52
         */
        fun getVersionCode(context: Context?): Int {
            var versionCode = 0
            val pm = context!!.packageManager
            try {
                val pi = pm.getPackageInfo(context.packageName, 0)
                versionCode = pi.versionCode
            } catch (e: PackageManager.NameNotFoundException) {
            }

            return versionCode
        }


        /**
         * 获取当前软件版本号
         *
         * @param context
         * @return
         * @author xxj
         * @time 2012-2-28 | 下午02:13:52
         */
        fun getVersionName(context: Context?): String {
            var versionName = ""
            val pm = context!!.packageManager
            try {
                val pi = pm.getPackageInfo(context.packageName, 0)
                versionName = pi.versionName
            } catch (e: PackageManager.NameNotFoundException) {
            }

            return versionName
        }

        /**
         * Get device name, manufacturer + model
         *
         * @return device name
         */
        val deviceName: String
            get() {
                val manufacturer = Build.MANUFACTURER
                val model = Build.MODEL

                return if (model.startsWith(manufacturer)) {
                    StrUtils.capitalize(model)
                } else {
                    StrUtils.capitalize(manufacturer) + " " + model
                }
            }

        /**
         * To determine whether it contains a gyroscope
         *
         * @return
         */
        fun isHaveGravity(context: Context): Boolean {
            val manager = context.getSystemService(Context.SENSOR_SERVICE) as SensorManager
            return manager != null
        }
    }

}
