package com.cc.tools.utils

import android.content.Context
import android.content.SharedPreferences
import android.net.ConnectivityManager
import android.net.Uri
import android.telephony.TelephonyManager


object NetworkUtils {

    //	public static final int IDLE = -1;
    val NOT_AVALBLE = 0
    val AVALBLE = 1
    //	public static final int WIFI = NOT_AVALBIE + 1;
    //	public static final int MOBILE_2G = WIFI + 1;
    //	public static final int MOBILE_3G = MOBILE_2G + 1;


    val NET_WORK_FILE = "new_work_file"
    val NET_WORK_STATE = "new_work_state_"

    val carriersContentUri: Uri
        @Throws(ClassNotFoundException::class, SecurityException::class, IllegalArgumentException::class, NoSuchFieldException::class, IllegalAccessException::class, InstantiationException::class)
        get() = getStaticField("android.provider.Telephony\$Carriers", "CONTENT_URI") as Uri

    val carriersApn: String
        @Throws(SecurityException::class, IllegalArgumentException::class, NoSuchFieldException::class, IllegalAccessException::class, ClassNotFoundException::class, InstantiationException::class)
        get() = getStaticField("android.provider.Telephony\$Carriers", "APN") as String

    val carriersProxy: String
        @Throws(SecurityException::class, IllegalArgumentException::class, NoSuchFieldException::class, IllegalAccessException::class, ClassNotFoundException::class, InstantiationException::class)
        get() = getStaticField("android.provider.Telephony\$Carriers", "PROXY") as String

    /**
     * 获取当前网络类型
     *
     * @param context 系统上下文，请使用[NduoaMarketApp.get]}获取Context
     * @return 网络类型
     * @time 2012-3-7 | 上午11:17:59
     */
    fun getConnectType(context: Context): Int {
        val cm = context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        val networkInfo = cm.activeNetworkInfo
        return if (networkInfo != null && networkInfo.isConnectedOrConnecting) {
            networkInfo.type
        } else -1
    }

    fun getDetailConnectType(context: Context): Int {
        val cm = context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        val networkInfo = cm.activeNetworkInfo
        if (networkInfo != null && networkInfo.isConnectedOrConnecting) {
            val extraInfo = networkInfo.extraInfo
            if (extraInfo == null) {
                //the connected is Wifi
            } else if (extraInfo == "cmnet") {
                //用移动CMNET方式
            } else if (extraInfo == "cmwap") {
                //用移动CMWAP方式,但是不在代理的情况下访问普通的网站访问不了
            } else if (extraInfo == "3gwap") {
                //用联通3gwap方式
            } else if (extraInfo == "3gnet") {
                //用联通3gnet方式
            } else if (extraInfo == "uniwap") {
                //用联通uniwap方式
            } else if (extraInfo == "uninet") {
                //用联通uninet方式
            }
        }
        return -1
    }

    /**
     * 判断网络是否可用（做延迟判断）
     *
     * @param context ： 系统上下文
     * @return 返回网络可用状态 true : 可用 ; false : 不可用
     * @time 2012-3-7 | 上午11:15:06
     */
    fun isNetworkAvailable(context: Context): Boolean {
        var result = __isNetworkAvailable(context)
        if (!result) {
            try {
                Thread.sleep(2000)
            } catch (e: InterruptedException) {
                result = false
            }

            result = __isNetworkAvailable(context)
        }
        return result
    }

    /**
     * 判断网络是否可用
     *
     * @param context ： 系统上下文
     * @return 返回网络可用状态 true : 可用 ; false : 不可用
     * @time 2012-3-7 | 上午11:17:25
     */
    fun __isNetworkAvailable(context: Context): Boolean {
        val cm = context
                .getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        val info = cm.activeNetworkInfo
        return info != null && info.isConnected
    }

    fun isWifiActive(context: Context): Boolean {
        val connectivity = context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        if (connectivity != null) {
            val info = connectivity.allNetworkInfo
            if (info != null) {
                for (i in info.indices) {
                    if (info[i].typeName == "WIFI" && info[i].isConnected) {
                        return true
                    }
                }
            }
        }
        return false
    }

    /**
     * 获取网络代理地址
     *
     * @param context ： 系统上下文
     * @return 返回代理地址
     * @time 2012-3-7 | 上午11:28:32
     */
    fun getProxyAddr(context: Context): String? {
        val connMgr = context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        val _info = connMgr.activeNetworkInfo
        if (_info != null) {
            try {
                val apn = _info.extraInfo
                if (apn != null) {
                    val c = context.contentResolver.query(carriersContentUri, null, null, null, null)

                    if (c!!.moveToFirst()) {
                        do {
                            val apnIdx = c.getColumnIndex(carriersApn)
                            val proxyIdx = c.getColumnIndex(carriersProxy)
                            val apnName = c.getString(apnIdx).toUpperCase()
                            val proxy = c.getString(proxyIdx)
                            if (apnName.contains(apn.toUpperCase())) {
                                return proxy
                            }
                        } while (c.moveToNext())
                    }
                }
            } catch (e: Exception) {
                return null
            }

        }

        val cursor = context.contentResolver.query(Uri.parse("content://telephony/carriers/preferapn"), null, null, null, null)
        if (cursor!!.moveToFirst()) {
            try {
                val proxyIdx = cursor.getColumnIndex(carriersProxy)
                return cursor.getString(proxyIdx)
            } catch (e: Exception) {
                return null
            }

        }
        return null
    }

    fun isFastMobileNetwork(context: Context): Boolean {
        val telephonyManager = context.getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
        when (telephonyManager.networkType) {
            TelephonyManager.NETWORK_TYPE_1xRTT -> return false // ~ 50-100 kbps
            TelephonyManager.NETWORK_TYPE_CDMA -> return false // ~ 14-64 kbps
            TelephonyManager.NETWORK_TYPE_EDGE -> return false // ~ 50-100 kbps
            TelephonyManager.NETWORK_TYPE_EVDO_0 -> return true // ~ 400-1000 kbps
            TelephonyManager.NETWORK_TYPE_EVDO_A -> return true // ~ 600-1400 kbps
            TelephonyManager.NETWORK_TYPE_GPRS -> return false // ~ 100 kbps
            TelephonyManager.NETWORK_TYPE_HSDPA -> return true // ~ 2-14 Mbps
            TelephonyManager.NETWORK_TYPE_HSPA -> return true // ~ 700-1700 kbps
            TelephonyManager.NETWORK_TYPE_HSUPA -> return true // ~ 1-23 Mbps
            TelephonyManager.NETWORK_TYPE_UMTS -> return true // ~ 400-7000 kbps
            TelephonyManager.NETWORK_TYPE_EHRPD -> return true // ~ 1-2 Mbps
            TelephonyManager.NETWORK_TYPE_EVDO_B -> return true // ~ 5 Mbps
            TelephonyManager.NETWORK_TYPE_HSPAP -> return true // ~ 10-20 Mbps
            TelephonyManager.NETWORK_TYPE_IDEN -> return false // ~25 kbps
            TelephonyManager.NETWORK_TYPE_LTE -> return true // ~ 10+ Mbps
            TelephonyManager.NETWORK_TYPE_UNKNOWN -> return false
            else -> return false
        }
    }

    fun updateNetWorkState(context: Context): Int {
        var status = NetworkUtils.AVALBLE
        if (NetworkUtils.__isNetworkAvailable(context)) {
            if (NetworkUtils.getConnectType(context) == ConnectivityManager.TYPE_WIFI) {
                //				ActivityDataUtils.setNetWorkState(context, NetworkUtils.WIFI);
            } else if (NetworkUtils.getConnectType(context) == ConnectivityManager.TYPE_MOBILE) {
                if (NetworkUtils.isFastMobileNetwork(context)) {
                    //					ActivityDataUtils.setNetWorkState(context, NetworkUtils.MOBILE_3G);
                } else {
                    //					ActivityDataUtils.setNetWorkState(context, NetworkUtils.MOBILE_2G);
                }
            }
            status = NetworkUtils.AVALBLE
        } else {
            status = NetworkUtils.NOT_AVALBLE
        }
        setNetWorkState(context, status)
        return status
    }

    @Throws(SecurityException::class, NoSuchFieldException::class, IllegalArgumentException::class, IllegalAccessException::class, ClassNotFoundException::class, InstantiationException::class)
    fun getStaticField(clsName: String, field: String): Any {
        val cls = Class.forName(clsName)
        val fd = cls.getDeclaredField(field)
        return fd.get(null)
    }

    fun setNetWorkState(context: Context, type: Int) {
        val preferences = context.getSharedPreferences(NET_WORK_FILE, Context.MODE_PRIVATE)
        preferences.edit().putInt(NET_WORK_STATE, type).commit()
    }

    fun registerNetWork(context: Context, listener: SharedPreferences.OnSharedPreferenceChangeListener) {
        val preferences = context.getSharedPreferences(NET_WORK_FILE, Context.MODE_PRIVATE)
        preferences.registerOnSharedPreferenceChangeListener(listener)
    }

    fun unregisterNetWork(context: Context, listener: SharedPreferences.OnSharedPreferenceChangeListener) {
        val preferences = context.getSharedPreferences(NET_WORK_FILE, Context.MODE_PRIVATE)
        preferences.unregisterOnSharedPreferenceChangeListener(listener)
    }

}
