package com.cc.tools.utils

import android.content.Context
import java.lang.reflect.InvocationTargetException
import java.lang.reflect.Method


object ReflectionUtils {

    //android.os.SystemProperties.get("ro.serialno");
    val androidID: String
        get() {
            try {
                val cls = Class.forName("android.os.SystemProperties")
                val getMtd = cls.getDeclaredMethod("get", String::class.java)
                return getMtd.invoke(null, "ro.serialno") as String
            } catch (e: Exception) {
                return "00000000"
            }

        }

    @Throws(SecurityException::class, NoSuchFieldException::class, IllegalArgumentException::class, IllegalAccessException::class, ClassNotFoundException::class, InstantiationException::class)
    fun getSystemService(context: Context, name: String): Any? {
        val service = getStaticField(Context::class.java.name, name)
        return context.getSystemService(service.toString())
    }

    @Throws(SecurityException::class, NoSuchFieldException::class, IllegalArgumentException::class, IllegalAccessException::class, ClassNotFoundException::class, InstantiationException::class)
    fun getStaticField(clsName: String, field: String): Any {
        val cls = Class.forName(clsName)
        val fd = cls.getDeclaredField(field)
        return fd.get(null)
    }

    @Throws(SecurityException::class, NoSuchFieldException::class, IllegalArgumentException::class, IllegalAccessException::class, ClassNotFoundException::class, InstantiationException::class)
    fun getStaticFieldInt(clsName: String, field: String): Int {
        val cls = Class.forName(clsName)
        val fd = cls.getDeclaredField(field)
        return fd.getInt(null)
    }

    @Throws(NoSuchFieldException::class, IllegalAccessException::class, ClassNotFoundException::class)
    fun getDeclaredField(o: Any, clsName: String, field: String): Any {
        return getDeclaredField(o, Class.forName(clsName), field)
    }

    @Throws(NoSuchFieldException::class, IllegalAccessException::class)
    fun getDeclaredField(o: Any, cls: Class<*>, field: String): Any {
        val fd = cls.getDeclaredField(field)
        fd.isAccessible = true
        return fd.get(o)
    }


    @Throws(ClassNotFoundException::class, NoSuchMethodException::class)
    fun getDeclaredMethod(clsName: String, methodName: String, vararg parameterTypes: Class<*>): Method {
        val cls = Class.forName(clsName)
        return getDeclaredMethod(cls, methodName, *parameterTypes)
    }

    @Throws(ClassNotFoundException::class, NoSuchMethodException::class)
    fun getDeclaredMethod(cls: Class<*>, methodName: String, vararg parameterTypes: Class<*>): Method {
        val method = cls.getDeclaredMethod(methodName, *parameterTypes)
        method.isAccessible = true
        return method
    }

    @Throws(InvocationTargetException::class, IllegalAccessException::class)
    fun invokeMothed(o: Any, method: Method, vararg args: Any): Any {
        method.isAccessible = true
        return method.invoke(o, *args)
    }

    //FileUtils.setPermissions(strApkPath, FileUtils.S_IRWXU
    //	| FileUtils.S_IRWXO | FileUtils.S_IRWXG, -1, -1);

    fun setFilePermissionsAllRWX(filePath: String) {
        try {
            val cls = Class.forName("android.os.FileUtils")
            val obj = cls.newInstance()
            val uRwx = cls.getDeclaredField("S_IRWXU")
            val oRwx = cls.getDeclaredField("S_IRWXO")
            val gRwx = cls.getDeclaredField("S_IRWXG")

            val mode = uRwx.getInt(obj) or oRwx.getInt(obj) or gRwx.getInt(obj)
            val setPerMtd = cls.getDeclaredMethod("setPermissions", String::class.java, Int::class.javaPrimitiveType, Int::class.javaPrimitiveType, Int::class.javaPrimitiveType)
            setPerMtd.invoke(null, filePath, mode, -1, -1)
        } catch (e: Exception) {
        }

    }

    fun buildFieldToString(o: Any): String {
        val buffer = StringBuffer()
        try {
            var aClass: Class<*>? = o.javaClass
            while (aClass != null) {
                val fields = aClass.declaredFields
                for (field in fields) {
                    field.isAccessible = true
                    buffer.append(field.name).append(":").append(field.get(o)).append("\n")
                }
                aClass = aClass.superclass
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }

        return buffer.toString()
    }

}
