package com.cc.tools.utils

import java.util.regex.Pattern


/**
 * 正则表达式
 *
 * @author kevin
 */
object RegexUtil {

    private fun isMatch(regex: String, orginal: String?): Boolean {
        if (orginal == null || orginal.trim { it <= ' ' } == "") {
            return false
        }
        val pattern = Pattern.compile(regex)
        val isNum = pattern.matcher(orginal)
        return isNum.matches()
    }

    fun isPositiveInteger(orginal: String): Boolean {
        return isMatch("^\\+{0,1}[1-9]\\d*", orginal)
    }

    fun isNegativeInteger(orginal: String): Boolean {
        return isMatch("^-[1-9]\\d*", orginal)
    }

    fun isWholeNumber(orginal: String): Boolean {
        return isMatch("[+-]{0,1}0", orginal) || isPositiveInteger(orginal) || isNegativeInteger(orginal)
    }

    fun isPositiveDecimal(orginal: String): Boolean {
        return isMatch("\\+{0,1}[0]\\.[1-9]*|\\+{0,1}[1-9]\\d*\\.\\d*", orginal)
    }

    fun isNegativeDecimal(orginal: String): Boolean {
        return isMatch("^-[0]\\.[1-9]*|^-[1-9]\\d*\\.\\d*", orginal)
    }

    fun isDecimal(orginal: String): Boolean {
        return isMatch("[-+]{0,1}\\d+\\.\\d*|[-+]{0,1}\\d*\\.\\d+", orginal)
    }

    fun isRealNumber(orginal: String): Boolean {
        return isWholeNumber(orginal) || isDecimal(orginal)
    }

    fun isEmail(email: String): Boolean {
        return isMatch("\\w+([-+.]\\w+)*@\\w+([-.]\\w+)*\\.\\w+([-.]\\w+)*", email)
    }

    fun isChinese(email: String): Boolean {
        return isMatch("^[\u4e00-\u9fa5]{0,}$", email)
    }

    fun isDate(date: String): Boolean {
        return isMatch(
                "^((\\d{2}(([02468][048])|([13579][26]))[\\-\\/\\s]?((((0?[13578])|(1[02]))[\\-\\/\\s]?((0?[1-9])|([1-2][0-9])|(3[01])))|(((0?[469])|(11))[\\-\\/\\s]?((0?[1-9])|([1-2][0-9])|(30)))|(0?2[\\-\\/\\s]?((0?[1-9])|([1-2][0-9])))))|(\\d{2}(([02468][1235679])|([13579][01345789]))[\\-\\/\\s]?((((0?[13578])|(1[02]))[\\-\\/\\s]?((0?[1-9])|([1-2][0-9])|(3[01])))|(((0?[469])|(11))[\\-\\/\\s]?((0?[1-9])|([1-2][0-9])|(30)))|(0?2[\\-\\/\\s]?((0?[1-9])|(1[0-9])|(2[0-8]))))))(\\s(((0?[0-9])|([1-2][0-3]))\\:([0-5]?[0-9])((\\s)|(\\:([0-5]?[0-9])))))?$",
                date)
    }

    fun isMobile(mobiles: String): Boolean {
        // return isMatch("^((13[0-9])|(15[^4,\\D])|(18[0,5-9]))\\d{8}$",
        // mobiles);
        return isMatch("^(1)\\d{10}$", mobiles)
    }

    fun checkString(regex: String, str: String): Boolean {
        return isMatch(regex, str)
    }

    @JvmStatic
    fun main(a: Array<String>) {
        // String text = "回家看看厉害吧[e_183][e_184][e_191][e_190][e_198]";
        // text = text.replaceAll("\\[e_([0-9]*)\\]",
        // "<img src='/img/smiley/e$1.gif' border='0' class='smiley' />");
        // System.out.println(text);

        println(isPositiveInteger("17466"))
    }
}
