package com.cc.tools.utils

import android.annotation.TargetApi
import android.app.Activity
import android.content.Context
import android.graphics.Rect
import android.os.Build
import android.view.View
import android.view.Window
import android.view.WindowManager
import com.cc.tools.R

object SystemBarUtils {

    var mActionBarHeight: Int = 0

    fun attachActivity(activity: Activity) {
        if (Build.VERSION.SDK_INT >= 21) {
            setTranslucentStatus(activity)
        } else if (Build.VERSION.SDK_INT >= 19) {
            setTranslucentStatus(activity, true)
        }
    }

    @TargetApi(21)
    fun setTranslucentStatus(activity: Activity) {
        val window = activity.window
        try {
            val FLAG_TRANSLUCENT_STATUS = getStaticFieldInt(WindowManager.LayoutParams::class.java.name, "FLAG_TRANSLUCENT_STATUS")
            window.clearFlags(FLAG_TRANSLUCENT_STATUS)
            window.addFlags(FLAG_TRANSLUCENT_STATUS)
            val SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN = getStaticFieldInt(View::class.java.name, "SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN")
            val SYSTEM_UI_FLAG_LAYOUT_STABLE = getStaticFieldInt(View::class.java.name, "SYSTEM_UI_FLAG_LAYOUT_STABLE")
            window.decorView.systemUiVisibility = SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN or SYSTEM_UI_FLAG_LAYOUT_STABLE
            val flag = getStaticFieldInt(WindowManager.LayoutParams::class.java.name, "FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS")
            window.addFlags(flag)
            val method = Window::class.java.getDeclaredMethod("setStatusBarColor", Int::class.javaPrimitiveType)
            method.invoke(window, activity.resources.getColor(R.color.color_transparent))
        } catch (e: Exception) {
            e.printStackTrace()
        }

    }

    @TargetApi(19)
    fun setTranslucentStatus(activity: Activity, on: Boolean) {
        val window = activity.window
        val winParams = window.attributes
        val bits: Int
        try {
            bits = getStaticFieldInt(WindowManager.LayoutParams::class.java.name, "FLAG_TRANSLUCENT_STATUS")
            if (on) {
                winParams.flags = winParams.flags or bits
            } else {
                winParams.flags = winParams.flags and bits.inv()
            }
            window.attributes = winParams
        } catch (e: Exception) {
            e.printStackTrace()
        }

    }

    @Throws(SecurityException::class, NoSuchFieldException::class, IllegalArgumentException::class, IllegalAccessException::class, ClassNotFoundException::class, InstantiationException::class)
    fun getStaticFieldInt(clsName: String, field: String): Int {
        val cls = Class.forName(clsName)
        val fd = cls.getDeclaredField(field)
        return fd.getInt(null)
    }

    fun initApplication(context: Context) {
        try {
            val c = Class.forName("com.android.internal.R\$dimen")
            val o = c.newInstance()
            val field = c.getField("status_bar_height")
            val x = field.get(o) as Int
            mActionBarHeight = context.resources.getDimensionPixelSize(x)
        } catch (e: Exception) {
            e.printStackTrace()
            if (context is Activity) {
                val frame = Rect()
                context.window.decorView.getWindowVisibleDisplayFrame(frame)
                mActionBarHeight = frame.top
            }
        }
    }

}
