package com.cc.tools.utils

import android.app.Activity
import android.app.Application
import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewConfiguration
import android.widget.EditText
import com.cc.tools.R


class ViewUtils {

    fun isActivityFinish(context: Context): Boolean {
        return (context as? Activity)?.isFinishing ?: false
    }

    fun inflateView(context: Context, resId: Int): View {
        val inflater = LayoutInflater.from(context)
        return inflater.inflate(resId, null)
    }

    private var mTouchSlop = 0
    private var mDownX = 0.0f
    private var mDownY = 0.0f

    fun initApplication(application: Application) {
        mTouchSlop = ViewConfiguration.get(application).scaledTouchSlop
    }

    fun downInfo(x: Float, y: Float) {
        mDownX = x
        mDownY = y
    }

    //是否点击区域起来滑动最小区间
    fun isNotPointView(x: Float, y: Float): Boolean {
        return Math.abs(x - mDownX) > mTouchSlop || Math.abs(y - mDownY) > mTouchSlop
    }

    //是否是电话
    fun checkMobile(input: EditText, phone: String): Boolean {
        if (!RegexUtil.isMobile(phone)) {
            ToastUtils.showLongToast(input.context, R.string.toast_input_phone_error)
            IMMUtils.showSoftInput(input.context, input)
            return false
        }
        return true
    }

    //是否是电话
    fun checkAccount(input: EditText, phone: String): Boolean {
        if (phone.length < 6 || phone.length > 18) {
            ToastUtils.showLongToast(input.context, R.string.toast_input_phone_error)
            IMMUtils.showSoftInput(input.context, input)
            return false
        }
        return true
    }

    fun checkPwd(input: EditText, pwd: String): Boolean {
        if (input.length() < 6) {
            ToastUtils.showLongToast(input.context, R.string.toast_input_pwd_error)
            IMMUtils.showSoftInput(input.context, input)
            return false
        }
        return true
    }

    fun checkAuthCode(input: EditText, code: String): Boolean {
        if (code.length != 6) {
            ToastUtils.showLongToast(input.context, R.string.toast_input_code_error)
            IMMUtils.showSoftInput(input.context, input)
            return false
        }
        return true
    }

    companion object {

        private var mIns: ViewUtils? = null

        val ins: ViewUtils
            get() {
                synchronized(ViewUtils::class.java) {
                    if (mIns == null) {
                        mIns = ViewUtils()
                    }
                }
                return mIns!!
            }
    }
}
