package com.cc.tools.utils.helper

import android.graphics.*
import android.view.View
import com.cc.tools.utils.DisplayUtils


class RoundLayoutHelper(view: View) {

    private var roundRadius: Int = 0
    private val roundPath: Path
    private val rectF: RectF
    private var paint: Paint
    var mIsCircle = false

    init {
        roundRadius = DisplayUtils.dip2px(view.context, 5f)
        roundPath = Path()
        rectF = RectF()
        view.setWillNotDraw(false)
        paint = Paint(Paint.ANTI_ALIAS_FLAG)
        paint.color = Color.WHITE
        view.setLayerType(View.LAYER_TYPE_NONE, paint)
    }

    fun setRoundRadius(roundRadius: Int) {
        this.roundRadius = roundRadius
    }

    fun draw(canvas: Canvas, saveCount: Int) {
        paint.xfermode = PorterDuffXfermode(PorterDuff.Mode.MULTIPLY)
        canvas.drawPath(roundPath, paint)
        canvas.restoreToCount(saveCount)
        paint.xfermode = null
    }

    fun onLayout(width: Int, height: Int) {
        rectF.left = 0f
        rectF.top = 0f
        rectF.right = width.toFloat()
        rectF.bottom = height.toFloat()
        if (mIsCircle) {
            roundPath.addCircle(width / 2f, height / 2f, roundRadius.toFloat(), Path.Direction.CW)
        } else {
            roundPath.addRoundRect(rectF, roundRadius.toFloat(), roundRadius.toFloat(), Path.Direction.CW)
        }
    }

}
