package com.cc.tools.utils.user

import android.content.Context


object UserInfoKeeper {

    private val FILE_NAME = "User"
    private val USER_TOKEN = "user_token"
    private val WEIXIN_OPEN_ID = "weixin_open_id"
    private val USER_INFO = "user_info"
    private val USER_ID = "user_id"
    private val NEED_SHOW_CHG_PWD = "need_show_chg_pwd"//要提示修改密码

    /**
     * 写入token
     *
     * @param context
     * @param token
     */
    fun writeUserToken(context: Context, token: String) {
        val preferences = context.getSharedPreferences(FILE_NAME, Context.MODE_PRIVATE)
        preferences.edit().putString(USER_TOKEN, token).commit()
    }

    /**
     * 读取token
     *
     * @param context
     * @return
     */
    fun readUserToken(context: Context): String {
        val preferences = context.getSharedPreferences(FILE_NAME, Context.MODE_PRIVATE)
        return preferences.getString(USER_TOKEN, "")
    }

    /**
     * 写入weixinOpenId
     *
     * @param context
     * @param token
     */
    fun writeWeixinOpenId(context: Context, openId: String) {
        val preferences = context.getSharedPreferences(FILE_NAME, Context.MODE_PRIVATE)
        preferences.edit().putString(WEIXIN_OPEN_ID, openId).commit()
    }

    /**
     * 读取weixinOpenId
     *
     * @param context
     * @return
     */
    fun readWeixinOpenId(context: Context): String {
        val preferences = context.getSharedPreferences(FILE_NAME, Context.MODE_PRIVATE)
        return preferences.getString(WEIXIN_OPEN_ID, "")
    }

    /**
     * 写入首页用户信息
     *
     * @param context
     * @param token
     */
    fun writeHomeUserInfo(context: Context, userInfo: String) {
        val preferences = context.getSharedPreferences(FILE_NAME, Context.MODE_PRIVATE)
        preferences.edit().putString(USER_INFO, userInfo).commit()
    }

    /**
     * 读取首页用户信息
     *
     * @param context
     * @return
     */
    fun readHomeUserInfo(context: Context): String {
        val preferences = context.getSharedPreferences(FILE_NAME, Context.MODE_PRIVATE)
        return preferences.getString(USER_INFO, "")
    }

    /**
     * 写入用户ID
     *
     * @param context
     * @param token
     */
    fun writeUserId(context: Context, id: String) {
        val preferences = context.getSharedPreferences(FILE_NAME, Context.MODE_PRIVATE)
        preferences.edit().putString(USER_ID, id).commit()
    }

    /**
     * 读取用户ID
     *
     * @param context
     * @return
     */
    fun readUserId(context: Context): String {
        val preferences = context.getSharedPreferences(FILE_NAME, Context.MODE_PRIVATE)
        return preferences.getString(USER_ID, "")
    }

    /**
     * 退出登录清除用户缓存
     *
     * @param context
     */
    fun logout(context: Context) {
        val preferences = context.getSharedPreferences(FILE_NAME, Context.MODE_PRIVATE)
        preferences.edit().clear().commit()
    }

    /**
     * 读取用户ID
     *
     * @param context
     * @return
     */
    fun isNeedShowBindPhone(context: Context, name: String): Boolean {
        val preferences = context.getSharedPreferences(NEED_SHOW_CHG_PWD, Context.MODE_PRIVATE)
        val key = NEED_SHOW_CHG_PWD + "_" + name
        val flag = preferences.getInt(key, 0) == 0
        if (flag) {
            preferences.edit().putInt(key, 1).commit()
        }
        return flag
    }

}
