package com.cc.tools.utils.user


import android.content.Context
import android.os.Environment
import android.text.TextUtils
import com.cc.tools.R
import com.cc.tools.bean.user.UserInfoBean
import com.cc.tools.bean.user.UserLoginBean
import com.cc.tools.network.HttpClientInst
import com.cc.tools.network.HttpConstants
import com.cc.tools.observer.UserLogoutObserver
import com.cc.tools.utils.JSONUtils
import com.cc.tools.utils.Storage
import com.cc.tools.utils.ToastUtils
import com.cc.tools.utils.start.LoginModuleUtils
import org.json.JSONException
import org.json.JSONObject
import java.io.*
import java.net.URLConnection

/**
 * 用户相关
 *
 * @author cgh
 */
class UserManager {

    private var mContext: Context? = null
    var userToken: String? = null
        private set
    /**
     * @return
     */
    var userId: String? = null
        private set
    var ccUserName = ""
        private set
    var isNeedChangePwd: Boolean = false
        private set
    private var mNeedBindPhone: Boolean = false
    /**
     * 获取用户信息
     *
     * @return
     */
    var userInfo: UserInfoBean? = null
        private set
    /**
     * 是否登录
     *
     * @return
     */
    val isLogin: Boolean
        get() = !TextUtils.isEmpty(userToken)

    fun initUserManage(context: Context) {
        this.mContext = context
        this.userToken = UserInfoKeeper.readUserToken(context)
        this.userInfo = UserInfoBean()
        this.userId = UserInfoKeeper.readUserId(context)
        this.userInfo = UserInfoBean()
        try {
            this.userInfo = JSONUtils.parseObject(UserInfoKeeper.readHomeUserInfo(mContext!!), UserInfoBean::class.java)
        } catch (e: JSONException) {
            e.printStackTrace()
        }
    }

    /**
     * 写入token
     *
     * @param userToken
     */
    fun writeUserToken(userToken: String) {
        this.userToken = userToken
        UserInfoKeeper.writeUserToken(mContext!!, userToken)
    }

    /**
     * 写入用户ID
     */
    fun writeUserId(userId: String) {
        this.userId = userId
        UserInfoKeeper.writeUserId(mContext!!, userId)
    }

    /**
     * 写入首页用户信息
     *
     * @param item
     */
    fun setEntityHomeUserInfoBean(str: String) {
        userInfo = JSONUtils.parseObject(str, UserInfoBean::class.java)
        UserInfoKeeper.writeHomeUserInfo(mContext!!, str)
    }

    fun writeUserName(mAccount: String) {
        if (Storage.externalStorageAvailable()) {
            Thread(Runnable {
                var fos: FileOutputStream? = null
                try {
                    val file = File(Environment.getExternalStorageDirectory(), ".ccplay")
                    if (!file.exists()) {
                        file.createNewFile()
                    }
                    fos = FileOutputStream(file)
                    fos.write(mAccount.toByteArray())
                } catch (e: Exception) {
                } finally {
                    if (fos != null) {
                        try {
                            fos.close()
                        } catch (e: IOException) {
                            e.printStackTrace()
                        }

                        fos = null
                    }
                }
            }).start()
        }
    }

    fun readUserName(): String? {
        var result: String? = null
        var `is`: InputStream? = null
        var inputReader: InputStreamReader? = null
        var bufReader: BufferedReader? = null
        try {
            val file = File(Environment.getExternalStorageDirectory(), ".ccplay")
            if (file.exists()) {
                `is` = FileInputStream(file)
                inputReader = InputStreamReader(`is`)
                bufReader = BufferedReader(inputReader)
                result = bufReader.readLine()
            }
        } catch (e: Exception) {
        } finally {
            if (bufReader != null) {
                try {
                    bufReader.close()
                } catch (e: IOException) {
                    e.printStackTrace()
                }

                bufReader = null
            }

            if (inputReader != null) {
                try {
                    inputReader.close()
                } catch (e: IOException) {
                    e.printStackTrace()
                }

                inputReader = null
            }

            if (`is` != null) {
                try {
                    `is`.close()
                } catch (e: IOException) {
                    e.printStackTrace()
                }
                `is` = null
            }
        }
        return result
    }

    fun checkLogin(toast: String, later: Boolean): Boolean {
        var toast = toast
        if (!isLogin) {
            if (TextUtils.isEmpty(toast)) {
                toast = mContext!!.getString(R.string.toast_not_login_2_action)
            }
            ToastUtils.showLongToast(mContext!!, toast)
            LoginModuleUtils.startLoginActivity(mContext!!, "", later)
            return false
        }
        return true
    }

    fun logout() {
        userToken = ""
        userId = ""
        isNeedChangePwd = false
        ccUserName = ""
        userInfo = UserInfoBean()
        userInfo!!.clear()
        HttpClientInst.removeCookie()
        UserInfoKeeper.logout(mContext!!)
        UserLogoutObserver.inst.onLogOutSuccess()
    }

    companion object {

        private var mInst: UserManager? = null

        val inst: UserManager
            get() {
                synchronized(UserManager::class.java) {
                    if (mInst == null) {
                        mInst = UserManager()
                    }
                }
                return mInst!!
            }

        fun addClientHeader(context: Context, urlConnection: URLConnection) {
            val userToken = UserInfoKeeper.readUserToken(context)
            if (!TextUtils.isEmpty(userToken)) {
                val jsonObject = JSONObject()
                try {
                    jsonObject.put("authorization_token", userToken)
                } catch (e: JSONException) {
                    e.printStackTrace()
                }
                urlConnection.setRequestProperty(HttpConstants.HEADER_USER_INFO, jsonObject.toString())
            }
        }

        fun setUserLoginBean(userLoginBean: UserLoginBean?) {
            UserManager.inst.writeUserName(userLoginBean!!.username)
            UserManager.inst.writeUserId(userLoginBean!!.user_id)
            UserManager.inst.writeUserToken(userLoginBean!!.authorization_token)
        }
    }

}
