package com.cc.tools.widget.actionbar

import android.app.Activity
import android.content.Context
import android.util.AttributeSet
import android.view.Gravity
import android.view.Menu
import android.view.MenuInflater
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import android.widget.Toast
import com.cc.tools.R


open class TitleBarLayout : ActionBasicLayout {

    var mLeftIcon: ImageView? = null
    var mTitleTv: TextView? = null
    var mMenuLayout: ViewGroup? = null
    var mLeftIconRes = 0

    constructor(ctx: Context, set: AttributeSet) : super(ctx, set) {

    }

    override fun onFinishInflate() {
        super.onFinishInflate()
        mLeftIcon = findViewById(R.id.actionbar_title_layout_left_icon) as ImageView
        mTitleTv = findViewById(R.id.actionbar_title_layout_title) as TextView
        mLeftIcon?.setOnClickListener({
            if (R.drawable.tools_common_back == mLeftIconRes) {
                (context as Activity).finish()
            }
        })
        mMenuLayout = findViewById(R.id.actionbar_title_layout_menu_layout) as ViewGroup
    }

    override fun setLeftIcon(resIcon: Int) {
        mLeftIconRes = resIcon
        mLeftIcon?.setImageResource(resIcon)
    }

    override fun setTitle(resId: Int) {
        if (resId > 0) {
            mTitleTv?.setText(resId)
        }
    }

    override fun setTitle(title: CharSequence?) {
        mTitleTv?.text = title
    }

    override fun onCreateOptionsMenu(menu: Menu?, inflater: MenuInflater?) {
        mMenuLayout!!.removeAllViews()
        val size = menu!!.size()
        for (i in 0 until size) {
            val menuItem = menu!!.getItem(i)
            menuItem.actionView.setOnClickListener {
                mIMenuModel?.onOptionsItemSelected(menuItem)
            }
            menuItem.actionView.setOnLongClickListener { v ->
                val toast = Toast.makeText(context, menuItem.titleCondensed, Toast.LENGTH_SHORT)
                val location = IntArray(2)
                v.getLocationInWindow(location)
                toast.setGravity(Gravity.TOP or Gravity.LEFT, location[0], location[1] + v.getHeight())
                toast.show()
                true
            }
            mMenuLayout!!.addView(menuItem.actionView)
        }
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        var widthSize = MeasureSpec.getSize(widthMeasureSpec)
        var heightMeasureSpec = MeasureSpec.makeMeasureSpec(suggestedMinimumHeight, MeasureSpec.EXACTLY)
        measureChild(mLeftIcon!!, widthMeasureSpec, heightMeasureSpec)
        measureChild(mMenuLayout!!, widthMeasureSpec, heightMeasureSpec)

        var widthMeasureSpec = MeasureSpec.makeMeasureSpec(widthSize - Math.max(if (mLeftIconRes > 0) mLeftIcon!!.measuredWidth else 0, mMenuLayout!!.measuredWidth) * 2, MeasureSpec.EXACTLY)

        mTitleTv!!.measure(widthMeasureSpec, heightMeasureSpec)
//        measureChild(, widthMeasureSpec, heightMeasureSpec)

        setMeasuredDimension(widthSize, suggestedMinimumHeight + paddingTop)
    }

    override fun onLayout(changed: Boolean, l: Int, t: Int, r: Int, b: Int) {
        super.onLayout(changed, l, t, r, b)
        var left = 0
        var top = paddingTop
        var right = 0
        var bottom = height
        if (mLeftIconRes > 0) {
            right = left + mLeftIcon!!.measuredWidth
            mLeftIcon!!.layout(left, top, right, bottom)
        } else {
            mLeftIcon!!.layout(0, 0, 0, 0)
        }

        left = (width - mTitleTv!!.measuredWidth) / 2
        right = left + mTitleTv!!.measuredWidth
        mTitleTv!!.layout(left, top, right, bottom)

        left = width - mMenuLayout!!.measuredWidth
        right = left + mMenuLayout!!.measuredWidth
        mMenuLayout!!.layout(left, top, right, bottom)
    }

}