package com.cc.tools.widget.drawable

import android.animation.ObjectAnimator
import android.animation.ValueAnimator
import android.content.res.ColorStateList
import android.graphics.Canvas
import com.cc.tools.application.BaseApplication
import com.cc.tools.observer.CodeTimeObserver
import com.cc.tools.utils.DisplayUtils

class DrawableGetCode : DrawableCircleRect, CodeTimeObserver.OnCodeTimeDownAction {

    var mTextWidth = DisplayUtils.dip2px(BaseApplication.mApplication, 73f)
    var mHeight = DisplayUtils.dip2px(BaseApplication.mApplication, 25f)
    var mTextSiz = DisplayUtils.dip2px(BaseApplication.mApplication, 11f)
    var mTextNow = ""
    var mCurrentWidth = 0
    var mAnimation: ObjectAnimator? = null

    constructor() {

    }

    constructor(colorStateList: ColorStateList) : super(colorStateList) {
        CodeTimeObserver.instance.addListener(this)
    }

    constructor(color: Int) : super(color) {
    }

    override fun onTimeStart(str: String) {
        mCurrentWidth = mTextWidth
        mAnimation = ObjectAnimator()
        mAnimation?.duration = 500
        mAnimation?.setIntValues(mTextWidth, mHeight)
        mAnimation?.addUpdateListener { animation ->
            mCurrentWidth = animation.animatedValue.toString().toInt()
            invalidateSelf()
        }
        mAnimation?.start()
        mTextNow = str
    }

    override fun onTimeContinued(str: String) {
        mCurrentWidth = if (CodeTimeObserver.instance.CLICK_ABLE) mTextWidth else mHeight
        mTextNow = str
        invalidateSelf()
    }

    override fun onTimeEnd(str: String) {
        mCurrentWidth = mHeight
        mTextNow = str
        mAnimation = ObjectAnimator()
        mAnimation?.duration = 500
        mAnimation?.setIntValues(mHeight, mTextWidth)
        mAnimation?.addUpdateListener(ValueAnimator.AnimatorUpdateListener {

            animation ->
            mCurrentWidth = animation.animatedValue.toString().toInt()
            invalidateSelf()

        })
        mAnimation?.start()
    }

    override fun invalidateSelf() {
        bounds.left = bounds.right - intrinsicWidth
        mRectF.set(bounds)
        super.invalidateSelf()
    }

    override fun draw(canvas: Canvas) {
        super.draw(canvas)
        canvas.save()
        mPaint.color = 0xFFFFFFFF.toInt()
        mPaint.textSize = mTextSiz.toFloat()
        var x = bounds.left + (intrinsicWidth - mPaint.measureText(mTextNow)) / 2
        var y = (canvas.height - (mPaint.ascent() + mPaint.descent())) / 2
        canvas.drawText(mTextNow, x.toFloat(), y, mPaint)
    }

    override fun getIntrinsicWidth(): Int {
        return mCurrentWidth
    }

    override fun getIntrinsicHeight(): Int {
        return mHeight
    }

    override fun getState(): IntArray {
        if (!CodeTimeObserver.instance.CLICK_ABLE) {
            return intArrayOf(android.R.attr.state_selected)
        }
        return super.getState()
    }

    override fun onDestroy() {
        CodeTimeObserver.instance.removeListener(this)
    }
}