package com.cc.tools.widget.input

import android.content.Context
import android.graphics.Canvas
import android.graphics.Paint
import android.graphics.Rect
import android.graphics.drawable.Drawable
import android.util.AttributeSet
import android.view.Gravity
import android.view.MotionEvent
import android.widget.EditText
import com.cc.tools.R
import com.cc.tools.model.common.ICodeModel
import com.cc.tools.observer.CodeTimeObserver
import com.cc.tools.utils.DisplayUtils
import com.cc.tools.utils.ViewUtils
import com.cc.tools.widget.drawable.DrawableGetCode

/**
 * 验证码
 */
class CodeView : EditText {

    var mPaint = Paint(Paint.ANTI_ALIAS_FLAG)
    var mRect = Rect()
    var mPadding = 0

    var mBgNormal = 0
    var mBgSelect = 0

    var mDrawableGetCode: DrawableGetCode

    constructor(ctx: Context, set: AttributeSet) : super(ctx, set) {
        mPadding = DisplayUtils.dip2px(ctx, 0.5f)
        mBgNormal = ctx.resources.getColor(R.color.color_common_line)
        mBgSelect = ctx.resources.getColor(R.color.color_common_white)
        mDrawableGetCode = DrawableGetCode(resources.getColorStateList(R.color.color_common_brown))
        mDrawableGetCode.callback = this

        textSize = 13f
        setBackgroundDrawable(null)
        compoundDrawablePadding = DisplayUtils.dip2px(ctx, 18f)
        gravity = Gravity.CENTER.or(Gravity.LEFT)
    }

    override fun onDraw(canvas: Canvas) {
        super.onDraw(canvas)
        if (isSelected || isFocused) {
            mPaint.color = mBgSelect
        } else {
            mPaint.color = mBgNormal
        }
        canvas.drawRect(mRect, mPaint)
        mDrawableGetCode.draw(canvas)
    }

    override fun onLayout(changed: Boolean, left: Int, top: Int, right: Int, bottom: Int) {
        super.onLayout(changed, left, top, right, bottom)
        mRect.left = 0
        mRect.top = height - mPadding
        mRect.right = width
        mRect.bottom = height

        var l = width - mDrawableGetCode.intrinsicWidth
        var t = (height - mDrawableGetCode.intrinsicHeight) / 2
        var r = l + mDrawableGetCode.intrinsicWidth
        var b = t + mDrawableGetCode.intrinsicHeight
        mDrawableGetCode.setBounds(l, t, r, b)
    }

    var mClick: Boolean = false

    override fun onTouchEvent(event: MotionEvent): Boolean {
        var action = event.action
        var x = event.x
        var y = event.y
        if (CodeTimeObserver.instance.CLICK_ABLE) {
            when (action) {
                MotionEvent.ACTION_DOWN -> {
                    ViewUtils.ins.downInfo(x, y)
                    mClick = mDrawableGetCode.bounds.contains(x.toInt(), y.toInt())
                }
                MotionEvent.ACTION_MOVE -> {
                    if (ViewUtils.ins.isNotPointView(x, y) || !mClick || !mDrawableGetCode.bounds.contains(x.toInt(), y.toInt())) {
                        mClick = false
                    }
                }
                MotionEvent.ACTION_UP -> {
                    if (mClick) {
                        try {
                            (context as ICodeModel).sendCode()
                        } catch (e: Exception) {
                            e.printStackTrace()
                        }
                    }
                }
            }
        }
        return super.onTouchEvent(event)
    }

    override fun invalidateDrawable(drawable: Drawable) {
        super.invalidateDrawable(drawable)
        var dirty = drawable.bounds
        invalidate(dirty.left, dirty.top, dirty.right, dirty.bottom)
    }

    override fun onDetachedFromWindow() {
        super.onDetachedFromWindow()
        mDrawableGetCode.onDestroy()
    }
}
