package com.cc.tools.widget.input

import android.content.Context
import android.graphics.Canvas
import android.graphics.drawable.Drawable
import android.text.Editable
import android.text.TextUtils
import android.text.TextWatcher
import android.util.AttributeSet
import android.view.Gravity
import android.view.MotionEvent
import com.cc.tools.R
import com.cc.tools.utils.ViewUtils

open class InputView : NormalInputView, TextWatcher {

    var mDrawableClear: Drawable
    var mType: Int = 0

    constructor(ctx: Context, set: AttributeSet) : super(ctx, set) {
        mDrawableClear = ctx.resources.getDrawable(R.drawable.tools_icon_clear)
        gravity = Gravity.CENTER.or(Gravity.LEFT)

        var attrs = intArrayOf(R.attr.attr_input_type)
        var typeArray = ctx.obtainStyledAttributes(set, attrs)
        mType = typeArray.getInt(0, -1)
        typeArray.recycle()

        addTextChangedListener(this)
    }

    override fun onDraw(canvas: Canvas) {
        super.onDraw(canvas)
        drawClearDrawable(canvas)
    }

    open fun drawClearDrawable(canvas: Canvas) {
        if (!TextUtils.isEmpty(text)) {
            mDrawableClear.draw(canvas)
        }
    }

    override fun onLayout(changed: Boolean, left: Int, top: Int, right: Int, bottom: Int) {
        super.onLayout(changed, left, top, right, bottom)
        mDrawableClear.bounds.left = width - mDrawableClear.intrinsicWidth
        mDrawableClear.bounds.top = (height - mDrawableClear.intrinsicHeight) / 2
        mDrawableClear.bounds.right = mDrawableClear.bounds.left + mDrawableClear.intrinsicWidth
        mDrawableClear.bounds.bottom = mDrawableClear.bounds.top + mDrawableClear.intrinsicHeight
    }

    var mClear: Boolean = false

    override fun onTouchEvent(event: MotionEvent): Boolean {
        var action = event.action
        var x = event.x
        var y = event.y
        when (action) {
            MotionEvent.ACTION_DOWN -> {
                ViewUtils.ins.downInfo(x, y)
                mClear = mDrawableClear.bounds.contains(x.toInt(), y.toInt())
            }
            MotionEvent.ACTION_MOVE -> {
                if (ViewUtils.ins.isNotPointView(x, y) || !mClear || !mDrawableClear.bounds.contains(x.toInt(), y.toInt())) {
                    mClear = false
                }
            }
            MotionEvent.ACTION_UP -> {
                if (mClear) {
                    text.clear()
                }
            }
        }
        return super.onTouchEvent(event)
    }

    override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
    }

    override fun onTextChanged(text: CharSequence, start: Int, lengthBefore: Int, lengthAfter: Int) {

    }

    var mCurrentText = ""
    override fun afterTextChanged(s: Editable) {
        if (TYPE_PHONE == mType) {
            if (TextUtils.isEmpty(s.toString())) return;
            var text = s.toString().replace(" ", "")
            if (!s.toString().equals(mCurrentText)) {
                var builder = StringBuilder()
                if (text.length > 7) {
                    builder.append(text.substring(7))
                    builder.insert(0, " ")
                    text = text.substring(0, 7)
                }
                if (text.length > 3) {
                    builder.insert(0, text.substring(3, text.length))
                    builder.insert(0, " ")
                    text = text.substring(0, 3)
                }
                builder.insert(0, text.subSequence(0, text.length))
                mCurrentText = builder.toString()
                s.replace(0, s.length, builder.toString())
                return
            }
            isSelected = !text.startsWith("1")
        }
    }

    override fun onDetachedFromWindow() {
        super.onDetachedFromWindow()
        removeTextChangedListener(this)
    }

    companion object {
        val TYPE_PHONE = 1
        val TYPE_EMAIL = 2
    }

}