package com.cc.tools.widget.input

import android.content.Context
import android.graphics.Canvas
import android.graphics.Paint
import android.graphics.Rect
import android.util.AttributeSet
import android.view.Gravity
import android.widget.EditText
import com.cc.tools.R
import com.cc.tools.utils.DisplayUtils


open class NormalInputView : EditText {

    var mPaint = Paint(Paint.ANTI_ALIAS_FLAG)
    var mRect = Rect()
    var mPadding = 0

    var mBgNormal = 0
    var mBgSelect = 0

    constructor(ctx: Context, set: AttributeSet) : super(ctx, set) {
        mPadding = DisplayUtils.dip2px(ctx, 0.5f)
        mBgNormal = ctx.resources.getColor(R.color.color_common_line)
        mBgSelect = ctx.resources.getColor(R.color.color_common_white)

        textSize = 13f
        compoundDrawablePadding = DisplayUtils.dip2px(ctx, 18f)
        setBackgroundDrawable(null)
        gravity = Gravity.CENTER.or(Gravity.LEFT)
    }

    override fun onDraw(canvas: Canvas) {
        super.onDraw(canvas)
        drawRect(canvas)
    }

    open fun drawRect(canvas: Canvas) {
        if (isSelected || isFocused) {
            mPaint.color = mBgSelect
        } else {
            mPaint.color = mBgNormal
        }
        canvas.drawRect(mRect, mPaint)
    }

    override fun onLayout(changed: Boolean, left: Int, top: Int, right: Int, bottom: Int) {
        super.onLayout(changed, left, top, right, bottom)
        mRect.left = 0
        mRect.top = height - mPadding
        mRect.right = width
        mRect.bottom = height
    }

}