package com.cc.tools.widget.progress

import android.content.Context
import android.graphics.*
import android.util.AttributeSet
import android.widget.ProgressBar
import com.cc.tools.utils.DisplayUtils


class CustomProgressView(context: Context, attrs: AttributeSet) : ProgressBar(context, attrs) {

    private val mBgColor: Int
    private val mStartColor: Int
    private val mEndColor: Int
    private val mRadius: Float
    private var mMax: Int = 0
    private var mProgress: Int = 0

    private val mPaint = Paint(Paint.ANTI_ALIAS_FLAG)
    private val mRect = RectF()

    init {
        mRadius = DisplayUtils.dip2px(context, 1.5f).toFloat()
        mBgColor = -0xb0b0c
        mStartColor = -0x43aa
        mEndColor = -0x2698
    }

    @Synchronized override fun onDraw(canvas: Canvas) {
        canvas.save()
        mPaint.reset()
        mPaint.color = mBgColor
        mRect.right = width.toFloat()
        canvas.drawRoundRect(mRect, mRadius, mRadius, mPaint)
        if (mMax == 0) {
            mMax = 100
        }
        if (mProgress < 0 || mProgress > mMax) {
            mProgress = 0
        }
        mRect.right = mRect.right * mProgress / mMax
        mPaint.shader = LinearGradient(mRect.left, mRect.top, mRect.right, mRect.bottom, mStartColor, mEndColor, Shader.TileMode.CLAMP)
        canvas.drawRoundRect(mRect, mRadius, mRadius, mPaint)
        canvas.restore()
    }

    override fun setMax(max: Int) {
        mMax = max
    }

    override fun setProgress(progress: Int) {
        mProgress = progress
        invalidate()
    }

    override fun getProgress(): Int {
        return mProgress
    }

    override fun getMax(): Int {
        return mMax
    }

    override fun onLayout(changed: Boolean, left: Int, top: Int, right: Int, bottom: Int) {
        super.onLayout(changed, left, top, right, bottom)
        mRect.left = 0f
        mRect.top = 0f
        mRect.right = width.toFloat()
        mRect.bottom = height.toFloat()
    }
}
