package com.cc.tools.widget.reclyer

import android.content.Context
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup

/**
 * @param <T>
</T> */
open abstract class BaseViewAdapter<T> : RecyclerView.Adapter<BaseHolder<T>>() {

    protected var mBeans: MutableList<T>? = null
    protected var mScrolling: Boolean = false// 是否滑动,滑动不加载
    protected var mLayoutInflater: LayoutInflater? = null
    protected var mClear = true
    protected var mClickIndex = -1

    fun setBeans(mBeans: MutableList<T>): BaseViewAdapter<T> {
        this.mBeans = mBeans
        return this
    }

    fun addBeans(beanList: List<T>) {
        mBeans!!.addAll(beanList)
    }

    fun setClear(clear: Boolean): BaseViewAdapter<*> {
        this.mClear = clear
        return this
    }

    override fun getItemCount(): Int {
        return mBeans!!.size
    }

    fun releaseAdapter() {
        if (mBeans != null) {
            if (mClear) {
                mBeans!!.clear()
            }
            mBeans = null
        }
    }

    protected fun getItem(position: Int): T {
        return mBeans!![position]
    }

    override fun onBindViewHolder(viewHolder: BaseHolder<T>, position: Int) {
        try {
            viewHolder.setEntityData(getItem(position), position)
        } catch (e: Exception) {
            e.printStackTrace()
            try {
                throw Exception(e.message)
            } catch (e1: Exception) {
                e1.printStackTrace()
            }

        }

    }

    override fun onCreateViewHolder(viewGroup: ViewGroup, viewType: Int): BaseHolder<T> {
        val itemView = inflateView(viewGroup.context, getItemLayoutId(viewType), viewGroup)
        return getBaseHolder(itemView, viewType)
    }

    protected fun inflateView(context: Context, resId: Int, root: ViewGroup): View {
        if (mLayoutInflater == null) {
            mLayoutInflater = LayoutInflater.from(context)
        }
        return mLayoutInflater!!.inflate(resId, root, false)
    }

    abstract fun getBaseHolder(itemView: View, viewType: Int): BaseHolder<T>

    abstract fun getItemLayoutId(viewType: Int): Int

    /**
     * 是否滑动,滑动不加载
     */
    fun setScrolling(scrolling: Boolean) {
        this.mScrolling = scrolling
    }

    fun onBackPressed(): Boolean {
        return false
    }

    companion object {

        fun releaseAdapter(adapter: BaseViewAdapter<*>?) {
            adapter?.releaseAdapter()
        }
    }

}
