package com.cc.tools.widget.reclyer


import android.content.Context
import android.graphics.Rect
import android.graphics.drawable.Drawable
import android.support.v7.widget.GridLayoutManager
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.cc.tools.R
import com.cc.tools.widget.reclyer.header.FooterLayout
import com.cc.tools.widget.reclyer.header.HeaderAdapter
import com.cc.tools.widget.reclyer.header.HeaderLayout
import java.lang.ref.WeakReference

//class CustomRecyclerView : RecyclerView, SwipeChild {
class CustomRecyclerView : RecyclerView {

    var mHeaderLayout: HeaderLayout? = null
    var mFooterLayout: FooterLayout? = null
    var mHeaderAdapter: HeaderAdapter? = null
    var mAction: WeakReference<OnCustomRecyclerViewAction>? = null
//     var childHelper: SwipeChildHelper? = null
//     var setSwipeEnable: Boolean = false


    protected var mDividerDecoration: DividerDecoration? = null
    var mDividerWidth: Int = 0
    /**
     * 获取横向分隔线高度
     *
     * @return
     */
    var dividerHeight: Int = 0
        set

    val currentItem: Int
        get() {
            if (layoutManager is LinearLayoutManager) {
                val layoutManager = layoutManager as LinearLayoutManager
                if (layoutManager.orientation == LinearLayoutManager.HORIZONTAL) {
                    val currentItem = layoutManager.findFirstVisibleItemPosition()
                    if (currentItem == 0) {
                        val view = getLayoutManager().findViewByPosition(currentItem)
                        val rect = Rect()
                        view.getLocalVisibleRect(rect)
                        if (view.width == rect.width()) {
                            return 0
                        }
                    }
                }
            }
            return -1
        }

    constructor(context: Context) : super(context) {
    }

    constructor(context: Context, attrs: AttributeSet) : super(context, attrs) {
    }

    init {
//        childHelper = SwipeChildHelper().setSwipeChild(this)
        mHeaderAdapter = HeaderAdapter()
        mDividerDecoration = DividerDecoration()
        addItemDecoration(mDividerDecoration)
    }

//    fun setSetSwipeEnable(setSwipeEnable: Boolean) {
//        this.setSwipeEnable = setSwipeEnable
//    }

    /**
     * 设置横向分隔线
     *
     * @param horizontalDrawable
     */
    fun setHorizontalDrawable(horizontalDrawable: Drawable) {
        if (mDividerDecoration != null) {
            mDividerDecoration!!.setHorizontalDrawable(horizontalDrawable)
        }
    }

    /**
     * 设置纵向分隔线
     *
     * @param verticalDrawable
     */
    fun setVerticalDrawable(verticalDrawable: Drawable) {
        if (mDividerDecoration != null) {
            mDividerDecoration!!.setVerticalDrawable(verticalDrawable)
        }
    }

    fun setDividerHeightPx(px: Int) {
        dividerHeight = px
        if (mDividerDecoration != null) {
            mDividerDecoration!!.setDividerHeight(dividerHeight)
        }
    }

    /**
     * 设置横向分隔线高度
     *
     * @param dip
     */
    fun setDividerHeight(dip: Float) {
        dividerHeight = dip2px(context, dip)
        if (mDividerDecoration != null) {
            mDividerDecoration!!.setDividerHeight(dividerHeight)
        }
    }

    /**
     * 设置纵向分隔线宽度
     *
     * @param dip
     */
    fun setDividerWidth(dip: Float) {
        mDividerWidth = dip2px(context, dip)
        setDividerWidthPx(mDividerWidth)
    }

    fun setDividerWidthPx(px: Int) {
        if (mDividerDecoration != null) {
            mDividerDecoration!!.setDividerWidth(px)
        }
    }

    fun setHorizontalPadding(left: Int, right: Int) {
        if (mDividerDecoration != null) {
            mDividerDecoration!!.setHorizontalPadding(left, right)
        }
    }

    fun setDrawCenter(drawCenter: Boolean) {
        if (mDividerDecoration != null) {
            mDividerDecoration!!.setDrawCenter(drawCenter)
        }
    }

    fun setHasTopLine(hasTopLine: Boolean) {
        if (mDividerDecoration != null) {
            mDividerDecoration!!.setHasTopLine(hasTopLine)
        }
    }

    /**
     * 添加头部
     *
     * @param view
     */
    fun addHeaderView(view: View) {
        addHeaderView(view, -1)
    }

    /**
     * 添加头部
     *
     * @param child
     * @param index
     */
    fun addHeaderView(child: View, index: Int) {
        if (mHeaderLayout == null) {
            mHeaderLayout = LayoutInflater.from(context).inflate(R.layout.layout_recycle_header, this, false) as HeaderLayout
            mHeaderAdapter!!.setHeaderLayout(mHeaderLayout!!)
        }
        mHeaderLayout!!.addView(child, index)
        mHeaderAdapter!!.notifyDataSetChanged()
    }

    @JvmOverloads
    fun addFooterView(view: View, index: Int = -1) {
        if (mFooterLayout == null) {
            mFooterLayout = LayoutInflater.from(context).inflate(R.layout.layout_recycle_footer, this, false) as FooterLayout
            mHeaderAdapter!!.setFooterLayout(mFooterLayout!!)
        }
        mFooterLayout!!.addView(view, index)
        mHeaderAdapter!!.notifyDataSetChanged()
    }

    fun addFooterView(view: View, index: Int = -1, params: ViewGroup.LayoutParams) {
        if (mFooterLayout == null) {
            mFooterLayout = LayoutInflater.from(context).inflate(R.layout.layout_recycle_footer, null, false) as FooterLayout
            mHeaderAdapter!!.setFooterLayout(mFooterLayout!!)
        }
        mFooterLayout!!.addView(view, index, params)
        mHeaderAdapter!!.notifyDataSetChanged()
    }

    /**
     * 删除头部
     *
     * @param view
     */
    fun removeHeaderView(view: View) {
        if (mHeaderLayout != null) {
            mHeaderLayout!!.removeView(view)
            if (mHeaderAdapter != null) {
                mHeaderAdapter!!.notifyDataSetChanged()
            }
        }
    }

    /**
     * 删除尾部
     *
     * @param view
     */
    fun removeFooterView(view: View) {
        if (mFooterLayout != null) {
            mFooterLayout!!.removeView(view)
            if (mHeaderAdapter != null) {
                mHeaderAdapter!!.notifyDataSetChanged()
            }
        }
    }

    /**
     * 是否使用本身刷新机制
     */
    override fun invalidate() {
        if (mAction != null && mAction!!.get() != null) {
            mAction!!.get()!!.invalidate()
        } else {
            super.invalidate()
        }
    }

    override fun setAdapter(adapter: RecyclerView.Adapter<*>) {
        mHeaderAdapter!!.setContentAdapter(adapter)
        super.setAdapter(mHeaderAdapter)
        if (layoutManager is GridLayoutManager) {//用来确定gridlayout当有headerview和footerview的时候的每一项列数
            val gridLayoutManager = layoutManager as GridLayoutManager
            gridLayoutManager.spanSizeLookup = object : GridLayoutManager.SpanSizeLookup() {
                override fun getSpanSize(position: Int): Int {
                    return if (mHeaderAdapter!!.isHeaderView(position) || mHeaderAdapter!!.isBottomView(position)) gridLayoutManager.spanCount else 1
                }
            }
        }
    }

    override fun setLayoutDirection(layoutDirection: Int) {
        super.setLayoutDirection(layoutDirection)
    }

    override fun onLayout(changed: Boolean, l: Int, t: Int, r: Int, b: Int) {
        try {
            super.onLayout(changed, l, t, r, b)
        } catch (e: Exception) {
            e.printStackTrace()
        }

    }

    fun setOnCustomRecyclerViewAction(action: OnCustomRecyclerViewAction) {
        this.mAction = WeakReference(action)
    }

    interface OnCustomRecyclerViewAction {
        fun invalidate()
    }

//    override fun onTouchEvent(event: MotionEvent): Boolean {
//        var onTouchEvent = false
//        try {
//            if (setSwipeEnable) {
//                childHelper!!.onTouchEvent(event)
//            }
//            onTouchEvent = super.onTouchEvent(event)
//        } catch (e: Exception) {
//            e.printStackTrace()
//        }
//
//        return onTouchEvent
//    }
//
//    override fun onInterceptTouchEvent(e: MotionEvent): Boolean {
//        if (setSwipeEnable) {
//            childHelper!!.onTouchEvent(e)
//        }
//        return super.onInterceptTouchEvent(e)
//    }


    fun onScrollToClose(): Boolean {
        return currentItem == 0
    }

    companion object {

        fun dip2px(context: Context, dpValue: Float): Int {
            val scale = context.resources.displayMetrics.density
            return (dpValue * scale + 0.5f).toInt()
        }
    }

}
/**
 * 添加尾部
 *
 * @param view
 */
