package com.cc.tools.widget.reclyer

import android.graphics.Canvas
import android.graphics.Rect
import android.graphics.drawable.Drawable
import android.support.v7.widget.GridLayoutManager
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.OrientationHelper
import android.support.v7.widget.RecyclerView
import android.view.View
import com.cc.tools.widget.reclyer.header.FooterLayout
import com.cc.tools.widget.reclyer.header.HeaderAdapter
import com.cc.tools.widget.reclyer.header.HeaderLayout

class DividerDecoration : RecyclerView.ItemDecoration() {

    private var mHorizontalDrawable: Drawable? = null
    private var mVerticalDrawable: Drawable? = null

    private var mDividerWidth: Int = 0
    private var mDividerHeight: Int = 0
    private var mDrawCenter: Boolean = false
    private var mHasTopLine: Boolean = false
    private var mHasBottomLine: Boolean = false

    private var mHorizontalPaddingLeft: Int = 0
    private var mHorizontalPaddingRight: Int = 0

    init {
        mHasTopLine = true
        mHasBottomLine = true
    }

    fun setDividerHeight(dividerHeight: Int) {
        this.mDividerHeight = dividerHeight
    }

    fun setDividerWidth(dividerWidth: Int) {
        this.mDividerWidth = dividerWidth
    }

    /**
     * 是否不画外围部分
     *
     * @param drawCenter
     */
    fun setDrawCenter(drawCenter: Boolean) {
        this.mDrawCenter = drawCenter
    }

    fun setHasTopLine(hasTopLine: Boolean) {
        mHasTopLine = hasTopLine
    }

    fun setHasBottomLine(hasBottomLine: Boolean) {
        mHasBottomLine = hasBottomLine
    }

    override fun onDraw(c: Canvas, parent: RecyclerView, state: RecyclerView.State?) {
        val layoutManager = parent.layoutManager
        if (layoutManager is GridLayoutManager) {
            drawGridLayoutDrawable(c, parent, layoutManager.spanCount)
        } else if (layoutManager is LinearLayoutManager) {
            if (LinearLayoutManager.VERTICAL == layoutManager.orientation) {
                drawHorizontalDrawable(c, parent, mHorizontalDrawable)
            } else {
                drawVerticalDrawable(c, parent, mVerticalDrawable)
            }
        }
    }

    fun setHorizontalDrawable(horizontalDrawable: Drawable) {
        this.mHorizontalDrawable = horizontalDrawable
    }


    fun setVerticalDrawable(verticalDrawable: Drawable) {
        this.mVerticalDrawable = verticalDrawable
    }

    fun setHorizontalPadding(left: Int, right: Int) {
        mHorizontalPaddingLeft = left
        mHorizontalPaddingRight = right
    }

    private fun drawHorizontalDrawable(canvas: Canvas, parent: RecyclerView, drawable: Drawable?) {
        if (drawable == null) return
        val left = parent.paddingLeft + mHorizontalPaddingLeft
        val right = parent.width - parent.paddingRight - mHorizontalPaddingRight
        val arr = getArr(parent)
        var i = arr[0]
        val count = arr[1]
        while (i < count) {
            val child = parent.getChildAt(i)
            val layoutParams = child.layoutParams as RecyclerView.LayoutParams
            val top = child.bottom + layoutParams.bottomMargin
            val bottom = top + mDividerHeight
            drawable.setBounds(left, top, right, bottom)
            drawable.draw(canvas)
            i++
        }
    }

    private fun drawVerticalDrawable(canvas: Canvas, parent: RecyclerView, drawable: Drawable?) {
        if (drawable == null) return
        val top = parent.paddingTop
        val bottom = parent.height - parent.paddingBottom
        val arr = getArr(parent)
        var i = arr[0]
        val count = arr[1]
        while (i < count) {
            val child = parent.getChildAt(i)
            val layoutParams = child.layoutParams as RecyclerView.LayoutParams
            val left = child.right + layoutParams.rightMargin
            val right = left + mDividerWidth
            drawable.setBounds(left, top, right, bottom)
            drawable.draw(canvas)
            i++
        }
    }

    private fun getArr(parent: RecyclerView): IntArray {
        val arr = IntArray(2)
        var firstPosition = -1
        val childCount = parent.childCount
        var lastPosition = -1
        val itemCount = parent.layoutManager.itemCount
        if (parent.layoutManager is LinearLayoutManager) {
            val layoutManager = parent.layoutManager as LinearLayoutManager
            firstPosition = layoutManager.findFirstVisibleItemPosition()
            lastPosition = layoutManager.findLastVisibleItemPosition()
        }
        val adapter = parent.adapter
        var count = 0
        var i = 0
        if (adapter is HeaderAdapter) {
            val headerCount = adapter.headerCount
            val footerCount = adapter.footerCount
            if (!mHasTopLine && firstPosition == 0) {
                i = 1
                count += headerCount
            }

            if (lastPosition == itemCount - 1) {
                count += footerCount
            }
        }

        arr[0] = i
        arr[1] = childCount - count
        return arr
    }

    private fun drawGridLayoutDrawable(canvas: Canvas, parent: RecyclerView, spanCount: Int) {
        var position = -1
        if (parent.layoutManager is LinearLayoutManager) {
            val layoutManager = parent.layoutManager as LinearLayoutManager
            position = layoutManager.findFirstVisibleItemPosition()
        }

        val adapter = parent.adapter
        var count = 0
        var index = 0
        if (adapter is HeaderAdapter) {
            count = adapter.footerCount + adapter.headerCount
        }

        if (mDrawCenter && position == 0) {
            index = 1
        }

        val childCount = parent.childCount - count
        var size = childCount / spanCount
        if (childCount % spanCount != 0) {
            size = size + 1
        }

        var left = 0
        var top = 0
        var right = 0
        var bottom = 0

        //	   Log.i("Logger", "index:" + index);
        if (mHorizontalDrawable != null) {
            left = 0
            right = parent.width
            if (!mDrawCenter) {
                top = 0
                bottom = top + parent.paddingTop
                mHorizontalDrawable!!.setBounds(left, top, right, bottom)
                mHorizontalDrawable!!.draw(canvas)

                top = parent.height - parent.paddingBottom
                bottom = top + parent.paddingBottom
                mHorizontalDrawable!!.setBounds(left, top, right, bottom)
                mHorizontalDrawable!!.draw(canvas)
            }

            for (i in index until size) {
                val child = parent.getChildAt(i * spanCount) ?: break
                val layoutParams = child.layoutParams as RecyclerView.LayoutParams


                if (mDrawCenter && i == 0) {

                } else {
                    top = child.top - layoutParams.topMargin - mDividerHeight
                    bottom = top + mDividerHeight
                    mHorizontalDrawable!!.setBounds(left, top, right, bottom)
                    mHorizontalDrawable!!.draw(canvas)
                }

                if (mDrawCenter && i == size - 1) {

                } else {
                    top = child.bottom + layoutParams.bottomMargin
                    bottom = top + mDividerHeight
                    mHorizontalDrawable!!.setBounds(left, top, right, bottom)
                    mHorizontalDrawable!!.draw(canvas)
                }
            }
        }

        if (mVerticalDrawable != null) {
            top = 0
            bottom = parent.height
            if (!mDrawCenter) {
                left = 0
                right = parent.paddingLeft
                mVerticalDrawable!!.setBounds(left, top, right, bottom)
                mVerticalDrawable!!.draw(canvas)

                left = parent.width - parent.paddingRight
                right = left + parent.paddingRight
                mVerticalDrawable!!.setBounds(left, top, right, bottom)
                mVerticalDrawable!!.draw(canvas)
            }

            for (i in index until spanCount) {
                val child = parent.getChildAt(i) ?: break
                val layoutParams = child.layoutParams as RecyclerView.LayoutParams
                if (mDrawCenter && i == 0) {

                } else {
                    left = child.left - layoutParams.leftMargin - mDividerWidth
                    right = left + mDividerWidth
                    mVerticalDrawable!!.setBounds(left, top, right, bottom)
                    mVerticalDrawable!!.draw(canvas)
                }

                if (mDrawCenter && i == size - 1) {

                } else {
                    left = child.right + layoutParams.rightMargin
                    right = left + mDividerWidth
                    mVerticalDrawable!!.setBounds(left, top, right, bottom)
                    mVerticalDrawable!!.draw(canvas)
                }
            }
        }
    }

    override fun getItemOffsets(outRect: Rect, view: View, parent: RecyclerView, state: RecyclerView.State?) {
        val layoutManager = parent.layoutManager
        if (layoutManager is GridLayoutManager) {
            outRect.set(mDividerWidth, mDividerHeight, mDividerWidth, mDividerHeight)//注意这里的偏移指的是view的left,top,right,bottom
            if (mDrawCenter) {
                //由于每行都只有3个，所以第一个都是3的倍数，把左边距设为0
                if (view is HeaderLayout) {
                    outRect.left = 0
                    outRect.top = 0
                    outRect.right = 0
                } else if (view is FooterLayout) {
                    outRect.left = 0
                    outRect.top = 0
                    outRect.bottom = 0
                } else {
                    var position = parent.getChildLayoutPosition(view)
                    var contentCount = parent.adapter.itemCount
                    var headerCount = 0
                    var footerCount = 0
                    if (parent.adapter is HeaderAdapter) {
                        val headerAdapter = parent.adapter as HeaderAdapter
                        headerCount = headerAdapter.headerCount
                        footerCount = headerAdapter.footerCount
                    }
                    position -= headerCount
                    contentCount -= headerCount
                    val spanCount = layoutManager.spanCount
                    if (headerCount == 0 && position < spanCount) {
                        outRect.top = 0
                    }
                    if (position % spanCount == 0) {
                        outRect.left = 0
                    }
                    if (position % spanCount == spanCount - 1) {
                        outRect.right = 0
                    }

                    if (footerCount == 0 && position >= contentCount / spanCount * spanCount) {
                        outRect.bottom = 0
                    }
                }
            }
        } else if (layoutManager is LinearLayoutManager) {
            val itemCount = layoutManager.itemCount
            val position = parent.getChildLayoutPosition(view)

            val contentCount = parent.adapter.itemCount
            var headerCount = 0
            var footerCount = 0
            if (parent.adapter is HeaderAdapter) {
                val headerAdapter = parent.adapter as HeaderAdapter
                headerCount = headerAdapter.headerCount
                footerCount = headerAdapter.footerCount
            }

            if (!mHasTopLine && headerCount > 0 && position == 0) {
                outRect.set(0, 0, 0, 0)
            } else {
                if (footerCount > 0 && position == itemCount - 1) {
                    outRect.set(0, 0, 0, 0)
                } else {
                    if (OrientationHelper.VERTICAL == layoutManager.orientation) {
                        outRect.set(0, 0, 0, mDividerHeight)
                    } else {
                        outRect.set(0, 0, mDividerWidth, 0)
                    }
                }
            }
        }
    }
}