package com.cc.tools.widget.reclyer.header

import android.support.v7.widget.RecyclerView
import android.support.v7.widget.RecyclerView.AdapterDataObserver
import android.view.View
import android.view.ViewGroup

class HeaderAdapter : RecyclerView.Adapter<RecyclerView.ViewHolder>() {

    private var mHeaderLayout: HeaderLayout? = null
    private var mFooterLayout: FooterLayout? = null
    private var mAdapter: RecyclerView.Adapter<*>? = null

    val headerCount: Int
        get() = if (mHeaderLayout != null) 1 else 0

    private val adapterCount: Int
        get() = if (mAdapter != null) mAdapter!!.itemCount else 0

    val footerCount: Int
        get() = if (mFooterLayout != null) 1 else 0

    fun setHeaderLayout(headerLayout: HeaderLayout) {
        this.mHeaderLayout = headerLayout
    }

    fun setFooterLayout(footerLayout: FooterLayout) {
        this.mFooterLayout = footerLayout
    }

    fun setContentAdapter(adapter: RecyclerView.Adapter<*>) {
        this.mAdapter = adapter
    }

    override fun getItemCount(): Int {
        return headerCount + adapterCount + footerCount
    }

    override fun onCreateViewHolder(viewGroup: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        when (viewType) {
            TYPE_CONTENT -> {
                return if (mAdapter != null) {
                    mAdapter!!.onCreateViewHolder(viewGroup, viewType - TYPE_CONTENT)
                } else HeaderHolder(View(viewGroup.context), this)
            }
            TYPE_HEADER -> return HeaderHolder(mHeaderLayout!!, this)
            TYPE_FOOTER -> return HeaderHolder(mFooterLayout!!, this)
            else -> {
                return if (mAdapter != null) {
                    mAdapter!!.onCreateViewHolder(viewGroup, viewType - TYPE_CONTENT)
                } else HeaderHolder(View(viewGroup.context), this)
            }
        }
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        var position = position
        val itemViewType = getItemViewType(position)
        position -= headerCount
        if (TYPE_CONTENT <= itemViewType) {
            if (mAdapter != null) {
                (mAdapter as RecyclerView.Adapter<RecyclerView.ViewHolder>)!!.onBindViewHolder(holder, position)
            }
        }
    }

    override fun getItemViewType(position: Int): Int {
        return if (position < headerCount) {
            TYPE_HEADER
        } else if (position >= headerCount + adapterCount) {
            TYPE_FOOTER
        } else
            mAdapter!!.getItemViewType(position - headerCount) + TYPE_CONTENT
    }

    fun isHeaderView(position: Int): Boolean {
        return headerCount != 0 && position < headerCount
    }

    //判断当前item是否是FooterView
    fun isBottomView(position: Int): Boolean {
        return footerCount != 0 && position >= headerCount + adapterCount
    }

    override fun registerAdapterDataObserver(observer: AdapterDataObserver) {
        if (mAdapter != null) {
            mAdapter!!.registerAdapterDataObserver(observer)
        }
        super.registerAdapterDataObserver(observer)
    }

    override fun unregisterAdapterDataObserver(observer: AdapterDataObserver) {
        if (mAdapter != null) {
            mAdapter!!.unregisterAdapterDataObserver(observer)
        }
        super.unregisterAdapterDataObserver(observer)
    }

    override fun getItemId(position: Int): Long {
        return position.toLong()
    }

    companion object {

        private val TYPE_HEADER = 0
        private val TYPE_FOOTER = TYPE_HEADER + 1
        private val TYPE_CONTENT = TYPE_FOOTER + 1
    }

}