package com.cc.tools.widget.text

import android.content.Context
import android.graphics.Canvas
import android.graphics.Paint
import android.graphics.RectF
import android.os.Handler
import android.util.AttributeSet
import android.widget.TextView
import com.cc.tools.R
import com.cc.tools.application.BaseApplication
import com.cc.tools.bean.CacheBean
import com.cc.tools.dlg.DlgCacheUse
import com.cc.tools.dlg.DlgUnZip
import com.cc.tools.model.common.IUnZipModel
import com.cc.tools.network.download.DownloadFileBean
import com.cc.tools.network.download.DownloadManager
import com.cc.tools.network.download.IDownloadProgress
import com.cc.tools.network.download.ZipUtils
import com.cc.tools.utils.HandlerUtils
import com.cc.tools.utils.ToastUtils
import java.io.File

open class DownloadText : TextView, IDownloadProgress {

    var mDownUrl: String = ""
    var mHandler: Handler = Handler()
    var mPaint = Paint(Paint.ANTI_ALIAS_FLAG)
    var mDownloadType: Int = -1
    var mRoundRect = RectF()
    var mPadding = 0f
    var mCurrent = 0L
    var mTotal = 100L
    var mDlgUnZip: DlgUnZip? = null
    var mDlgCacheUse: DlgCacheUse? = null
    var mDownloadFileBean: DownloadFileBean? = null

    constructor(ctx: Context) : super(ctx) {
        mPaint.color = resources.getColor(R.color.color_common_yellow)
        mPadding = resources.getDimension(R.dimen.common_dlg_radius)
    }

    constructor(ctx: Context, set: AttributeSet) : super(ctx, set) {
        mPaint.color = resources.getColor(R.color.color_common_yellow)
        mPadding = resources.getDimension(R.dimen.common_dlg_radius)
    }

    override fun onDraw(canvas: Canvas?) {
        when (mDownloadType) {
            DownloadManager.DOWNLOAD_ING, DownloadManager.DOWNLOAD_PAUSE -> {
                mRoundRect.right = (mCurrent * width / mTotal).toFloat()
                canvas?.drawRoundRect(mRoundRect, mPadding, mPadding, mPaint)
            }
        }
        super.onDraw(canvas)
    }

    override fun onLayout(changed: Boolean, left: Int, top: Int, right: Int, bottom: Int) {
        super.onLayout(changed, left, top, right, bottom)
        mRoundRect.left = 0f
        mRoundRect.top = 0f
        mRoundRect.right = width.toFloat()
        mRoundRect.bottom = height.toFloat()
    }

    open fun checkDownloadStatus(cacheBean: CacheBean) {
        var bean = DownloadManager.inst.getDownloadFileBean(cacheBean.archiveUrl)
        bean.totalBytes = cacheBean.archiveSize
        bean.url = cacheBean.archiveUrl
        bean.pkgName = cacheBean.packageName
        bean.title = cacheBean.archiveName
        bean.savePath = getCachePath(cacheBean)
        mDownloadFileBean = bean
        when (bean.state) {
            DownloadManager.DOWNLOAD_IDE -> {
                onDownloadIde(bean)
            }
            DownloadManager.DOWNLOAD_ING -> {
                onDownloadProgress(bean!!)
            }
            DownloadManager.DOWNLOAD_PAUSE -> {
                onDownloadPaused(bean!!)
            }
            DownloadManager.DOWNLOAD_ED -> {
                onDownloadEnd(bean!!)
            }
        }
        setOnClickListener({
            var bean = mDownloadFileBean!!
            when (bean.state) {
                DownloadManager.DOWNLOAD_IDE -> {
                    useDown(bean)
                }
                DownloadManager.DOWNLOAD_ING -> {
                    ToastUtils.showLongToast(context, "正在下载中...")
                }
                DownloadManager.DOWNLOAD_PAUSE -> {
                    useDown(bean)
                }
                DownloadManager.DOWNLOAD_ED -> {
                    checkDownload()
                }
            }
        })
    }

    fun getCachePath(cacheBean: CacheBean): String {
        return File(BaseApplication.mApplication!!.getExternalFilesDir(""), cacheBean.archiveName + "_" + cacheBean.id + "_" + cacheBean.updateDatetime).absolutePath
    }

    fun useDown(bean: DownloadFileBean) {
        var file = File(bean.savePath)
        if (bean.totalBytes == file.length()) {
            bean.state = DownloadManager.DOWNLOAD_ED
            DownloadManager.inst.writeDownloadFileBean(bean)
            checkDownload()
        } else {
            DownloadManager.inst.addDownload(bean.title, bean.url, bean.totalBytes, bean.pkgName, bean.savePath)
        }
    }

    fun checkDownload() {
        mDlgCacheUse?.dismiss()
        mDlgCacheUse = DlgCacheUse(context)
        mDlgCacheUse!!.mDownloadFileBean = mDownloadFileBean
        mDlgCacheUse!!.mListener = object : DlgCacheUse.OnDlgCacheUseListener {

            override fun onUse(bean: DownloadFileBean) {
                var file = File(bean.savePath)
                if (!file.exists()) {
                    ToastUtils.showLongToast(context, "文件不存在，正在重新下载!")
                    DownloadManager.inst.addDownload(bean.title, bean.url, bean.totalBytes, bean.pkgName, bean.savePath)
                } else {
                    ZipUtils.unZip(bean.savePath, object : IUnZipModel {

                        override fun onError() {
                            HandlerUtils.post(mHandler, Runnable {
                                mDlgUnZip?.dismiss()
                                ToastUtils.showLongToast(context, "文件解压失败~")
                            })
                        }

                        override fun onComplete() {
                            HandlerUtils.post(mHandler, Runnable {
                                mDlgUnZip?.dismiss()
                                ToastUtils.showLongToast(context, "存档释放成功，退出游戏再打开即可使用存档~")
                            })
                        }

                        override fun onProgress(progress: Long, total: Long) {
                            HandlerUtils.post(mHandler, Runnable {
                                if (mDlgUnZip == null) {
                                    mDlgUnZip = DlgUnZip(context)
                                }
                                mDlgUnZip?.show()
                                mDlgUnZip?.setPoint(progress, total)
                            })
                        }
                    })
                }
            }
        }
        mDlgCacheUse!!.show()
    }

    override fun onDownloadIde(bean: DownloadFileBean) {
        if (mDownUrl == bean.url) {
            mDownloadFileBean = bean
            HandlerUtils.post(mHandler, Runnable {
                mDownloadType = DownloadManager.DOWNLOAD_IDE
                text = resources.getString(R.string.text_download)
                setTextColor(resources.getColorStateList(R.color.color_selector_white_2_yellow))
                setBackgroundResource(R.drawable.drawable_selector_round)
            })
        }
    }

    override fun onDownloadProgress(bean: DownloadFileBean) {
        if (mDownUrl == bean.url) {
            mDownloadFileBean = bean
            HandlerUtils.post(mHandler, Runnable {
                mCurrent = bean.currentBytes
                mTotal = bean.totalBytes
                text = String.format("%.1f%%", bean.currentBytes * 100f / bean.totalBytes)
                setTextColor(resources.getColor(R.color.color_common_white))
                mDownloadType = bean.state
            })
        }
    }

    override fun onDownloadEnd(bean: DownloadFileBean) {
        if (mDownUrl == bean.url) {
            mDownloadFileBean = bean
            HandlerUtils.post(mHandler, Runnable {
                mCurrent = bean.currentBytes
                mTotal = bean.totalBytes
                text = resources.getString(R.string.text_use)
                setTextColor(resources.getColorStateList(R.color.color_selector_yellow_2_white))
                setBackgroundResource(R.drawable.drawable_selector_brown)
                mDownloadType = bean.state
            })
        }
    }

    override fun onDownloadPaused(bean: DownloadFileBean) {
        if (mDownUrl == bean.url) {
            mDownloadFileBean = bean
            HandlerUtils.post(mHandler, Runnable {
                mCurrent = bean.currentBytes
                mTotal = bean.totalBytes
                mDownloadType = bean.state
                text = resources.getString(R.string.text_download_goon)
                setTextColor(resources.getColorStateList(R.color.color_selector_white_2_yellow))
                setBackgroundResource(R.drawable.drawable_selector_round)
            })
        }
    }

    override fun onAttachedToWindow() {
        super.onAttachedToWindow()
        DownloadManager.inst.addListener(this)
    }

    override fun onDetachedFromWindow() {
        super.onDetachedFromWindow()
        DownloadManager.inst.removeListener(this)
        mDlgUnZip?.dismiss()
    }
}