package com.cc.tools.widget.text

import android.content.Context
import android.graphics.Canvas
import android.graphics.drawable.Drawable
import android.util.AttributeSet
import com.cc.tools.R
import com.cc.tools.application.BaseApplication
import com.cc.tools.bean.CacheBean
import com.cc.tools.dlg.DlgDownload
import com.cc.tools.network.download.DownloadFileBean
import com.cc.tools.network.download.DownloadManager
import com.cc.tools.utils.HandlerUtils
import com.cc.tools.utils.Logger
import com.cc.tools.utils.ToastUtils
import java.io.File

class UserDownloadText : DownloadText {

    var mDlgDownload: DlgDownload? = null
    var mDrawable: Drawable? = null

    constructor(ctx: Context) : super(ctx) {
        mDrawable = resources.getDrawable(R.drawable.tools_select)
    }

    constructor(ctx: Context, set: AttributeSet) : super(ctx, set) {
        mDrawable = resources.getDrawable(R.drawable.tools_select)
    }

    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        if (isSelected) {
            Logger.i(mDrawable!!)
            var left = width - paddingRight - mDrawable?.intrinsicWidth!!
            var top = (height - mDrawable?.intrinsicHeight!!) / 2
            var right = left + mDrawable?.intrinsicWidth!!
            var bottom = top + mDrawable?.intrinsicHeight!!
            mDrawable?.setBounds(left, top, right, bottom)
            mDrawable?.draw(canvas)
        }
    }

    override fun checkDownloadStatus(cacheBean: CacheBean) {
        var bean = DownloadManager.inst.getDownloadFileBean(cacheBean.archiveUrl)
        mDownUrl = bean.url
        bean.totalBytes = cacheBean.archiveSize
        bean.url = cacheBean.archiveUrl
        bean.pkgName = cacheBean.packageName
        bean.title = cacheBean.archiveName
        bean.savePath = getCachePath(cacheBean)
        mDownloadFileBean = bean
    }

    fun readCache() {
        var bean = mDownloadFileBean!!
        when (bean.state) {
            DownloadManager.DOWNLOAD_IDE -> {
                useDown(bean)
            }
            DownloadManager.DOWNLOAD_ING -> {
                ToastUtils.showLongToast(context, "正在下载中...")
            }
            DownloadManager.DOWNLOAD_PAUSE -> {
                useDown(bean)
            }
            DownloadManager.DOWNLOAD_ED -> {
                checkDownload()
            }
        }
    }

    override fun onDownloadPaused(bean: DownloadFileBean) {
        if (bean.url == mDownUrl) {
            HandlerUtils.post(mHandler, Runnable {
                ToastUtils.showLongToast(context, "文档下载失败~，请重试")
            })
        }
    }

    override fun onDownloadProgress(bean: DownloadFileBean) {
        if (bean.url == mDownUrl) {
            HandlerUtils.post(mHandler, Runnable {
                if (mDlgDownload == null) {
                    mDlgDownload = DlgDownload(context)
                }
                mDlgDownload?.show()
                mDlgDownload?.setPoint(bean.currentBytes, bean.totalBytes)
            })
        }
    }

    override fun onDownloadEnd(bean: DownloadFileBean) {
        if (bean.url == mDownUrl) {
            HandlerUtils.post(mHandler, Runnable {
                mDlgDownload?.dismiss()
                checkDownload()
            })
        }
    }

    override fun onDetachedFromWindow() {
        super.onDetachedFromWindow()
        mDlgDownload?.dismiss()
    }
}